@@ -26,17 +26,24 @@
 extern "C" {
 #endif
 
+/* WIN32, not Xbox */
 #ifdef _WIN32
-#define ALAPI       __declspec(dllexport)
-#define ALAPIENTRY  __cdecl
+#ifndef _XBOX
+#ifdef _OPENAL32LIB
+#define ALAPI __declspec(dllexport)
+#else
+#define ALAPI __declspec(dllimport)
+#endif
+#define ALAPIENTRY __cdecl
 #define AL_CALLBACK 
-#else /* _WIN32 */
+#endif
+#endif
 
 #ifdef TARGET_OS_MAC
 #if TARGET_OS_MAC
 #pragma export on
-#endif /* TARGET_OS_MAC */
-#endif /* TARGET_OS_MAC */
+#endif
+#endif
 
 #ifndef ALAPI
 #define ALAPI
@@ -50,7 +57,7 @@ extern "C" {
 #define AL_CALLBACK 
 #endif
 
-#endif /* _WIN32 */
+#define OPENAL
 
 #ifndef AL_NO_PROTOTYPES
 
@@ -126,7 +133,6 @@ ALAPI ALenum ALAPIENTRY alGetEnumValue( const ALubyte* ename );
 
 
 
-
 /**
  * LISTENER
  * Listener is the sample position for a given context.
@@ -160,15 +166,17 @@ ALAPI void ALAPIENTRY alListener3f( ALenum pname,
  * Listener Velocity:        ALfloat[3]
  * Listener Orientation:     ALfloat[6]  (forward and up vector).
  */
-ALAPI void ALAPIENTRY alListenerfv( ALenum pname, ALfloat* param ); 
+ALAPI void ALAPIENTRY alListenerfv( ALenum pname, const ALfloat* param ); 
 
 /*
  * Retrieve listener information.
  */
 ALAPI void ALAPIENTRY alGetListeneri( ALenum pname, ALint* value );
 ALAPI void ALAPIENTRY alGetListenerf( ALenum pname, ALfloat* value );
-
+#ifdef LINUX_AL
+/* not defined under Windows and MacOS */
 ALAPI void ALAPIENTRY alGetListeneriv( ALenum pname, ALint* value );
+#endif
 ALAPI void ALAPIENTRY alGetListenerfv( ALenum pname, ALfloat* values );
 
 ALAPI void ALAPIENTRY alGetListener3f( ALenum pname,
@@ -189,7 +197,7 @@ ALAPI void ALAPIENTRY alGetListener3f( ALenum pname,
 ALAPI void ALAPIENTRY alGenSources( ALsizei n, ALuint* sources ); 
 
 /** Delete Source objects. */
-ALAPI void ALAPIENTRY alDeleteSources( ALsizei n, ALuint* sources );
+ALAPI void ALAPIENTRY alDeleteSources( ALsizei n, const ALuint* sources );
 
 /** Verify a handle is a valid Source. */ 
 ALAPI ALboolean ALAPIENTRY alIsSource( ALuint sid ); 
@@ -200,11 +208,14 @@ ALAPI void ALAPIENTRY alSourcei( ALuint sid, ALenum param, ALint value );
 ALAPI void ALAPIENTRY alSourcef( ALuint sid, ALenum param, ALfloat value ); 
 ALAPI void ALAPIENTRY alSource3f( ALuint sid, ALenum param,
                                   ALfloat f1, ALfloat f2, ALfloat f3 );
-ALAPI void ALAPIENTRY alSourcefv( ALuint sid, ALenum param, ALfloat* values ); 
+ALAPI void ALAPIENTRY alSourcefv( ALuint sid, ALenum param, const ALfloat* values ); 
 
 /** Get an integer parameter for a Source object. */
 ALAPI void ALAPIENTRY alGetSourcei( ALuint sid,  ALenum pname, ALint* value );
+#ifdef LINUX_AL
+/* not defined under Windows and MacOS */
 ALAPI void ALAPIENTRY alGetSourceiv( ALuint sid,  ALenum pname, ALint* values );
+#endif
 ALAPI void ALAPIENTRY alGetSourcef( ALuint sid, ALenum pname, ALfloat* value );
 ALAPI void ALAPIENTRY alGetSourcefv( ALuint sid, ALenum pname, ALfloat* values );
 
@@ -212,10 +223,10 @@ ALAPI void ALAPIENTRY alGetSourcefv( ALuint sid, ALenum pname, ALfloat* values )
 ALAPI void ALAPIENTRY alGetSource3f( ALuint sid, ALenum pname, ALfloat* value1,
                                      ALfloat* value2, ALfloat* value3);
 
-ALAPI void ALAPIENTRY alSourcePlayv( ALsizei ns, ALuint *ids );
-ALAPI void ALAPIENTRY alSourceStopv( ALsizei ns, ALuint *ids );
-ALAPI void ALAPIENTRY alSourceRewindv( ALsizei ns, ALuint *ids );
-ALAPI void ALAPIENTRY alSourcePausev( ALsizei ns, ALuint *ids );
+ALAPI void ALAPIENTRY alSourcePlayv( ALsizei ns, const ALuint *ids );
+ALAPI void ALAPIENTRY alSourceStopv( ALsizei ns, const ALuint *ids );
+ALAPI void ALAPIENTRY alSourceRewindv( ALsizei ns, const ALuint *ids );
+ALAPI void ALAPIENTRY alSourcePausev( ALsizei ns, const ALuint *ids );
 
 /** Activate a source, start replay. */
 ALAPI void ALAPIENTRY alSourcePlay( ALuint sid );
@@ -257,7 +268,7 @@ ALAPI void ALAPIENTRY alSourceStop( ALuint sid );
 /** Buffer object generation. */
 ALAPI void ALAPIENTRY alGenBuffers( ALsizei n, ALuint* buffers );
 
-ALAPI void ALAPIENTRY alDeleteBuffers( ALsizei n, ALuint* buffers );
+ALAPI void ALAPIENTRY alDeleteBuffers( ALsizei n, const ALuint* buffers );
 
 
 ALAPI ALboolean ALAPIENTRY alIsBuffer( ALuint buffer );
@@ -267,14 +278,17 @@ ALAPI ALboolean ALAPIENTRY alIsBuffer( ALuint buffer );
  */
 ALAPI void ALAPIENTRY alBufferData( ALuint   buffer,
                    ALenum   format,
-                   ALvoid*    data,
+                   const ALvoid*    data,
                    ALsizei  size,
                    ALsizei  freq );
 
 ALAPI void ALAPIENTRY alGetBufferi( ALuint buffer, ALenum param, ALint*   value );
 ALAPI void ALAPIENTRY alGetBufferf( ALuint buffer, ALenum param, ALfloat* value );
+#ifdef LINUX_AL
+/* not defined under Windows and MacOS */
 ALAPI void ALAPIENTRY alGetBufferiv( ALuint buffer, ALenum param, ALint *v);
 ALAPI void ALAPIENTRY alGetBufferfv( ALuint buffer, ALenum param, ALfloat *v);
+#endif
 
 
 
@@ -291,8 +305,8 @@ ALAPI void ALAPIENTRY alGetBufferfv( ALuint buffer, ALenum param, ALfloat *v);
  */
 
 
-
-
+#ifdef LINUX_AL
+/* IASIG stuff never really used -- will probably be removed from Linux as well */
 /**
  * EXTENSION: IASIG Level 2 Environment.
  * Environment object generation.
@@ -300,10 +314,6 @@ ALAPI void ALAPIENTRY alGetBufferfv( ALuint buffer, ALenum param, ALfloat *v);
  *  properties according to IASIG Level 2 specifications.
  */
 
-
-
-
-
 /**
  * Allocate n environment ids and store them in the array environs.
  * Returns the number of environments actually allocated.
@@ -318,14 +328,18 @@ ALAPI void ALAPIENTRY alEnvironmentiIASIG( ALuint eid, ALenum param, ALint value
 
 ALAPI void ALAPIENTRY alEnvironmentfIASIG( ALuint eid, ALenum param, ALuint value );
 
-
+#endif
 
 /**
  * Queue stuff
  */
-ALAPI void ALAPIENTRY alSourceQueueBuffers( ALuint sid, ALsizei numEntries, ALuint *bids );
+ALAPI void ALAPIENTRY alSourceQueueBuffers( ALuint sid, ALsizei numEntries, const ALuint *bids );
 ALAPI void ALAPIENTRY alSourceUnqueueBuffers( ALuint sid, ALsizei numEntries, ALuint *bids );
+
+#ifdef LINUX_AL
+/* function no longer used */
 ALAPI void ALAPIENTRY alQueuei( ALuint sid, ALenum param, ALint value );
+#endif
 
 /**
  * Knobs and dials
@@ -355,7 +369,7 @@ ALAPI void ALAPIENTRY alDistanceModel( ALenum distanceModel );
                                                       ALfloat* data );
       void                         (*alGetDoublev)( ALenum param,
                                                       ALdouble* data );
-      const ALubyte*               (*GetString)( ALenum param );
+      const ALubyte*               (*alGetString)( ALenum param );
       ALenum                       (*alGetError)( ALvoid );
 
       /** 
@@ -370,7 +384,6 @@ ALAPI void ALAPIENTRY alDistanceModel( ALenum distanceModel );
        *  with the name fname. All addresses are context-independent. 
        */
       void*                (*alGetProcAddress)( const ALubyte* fname );
-      
 
       /**
        * Extension support.
@@ -398,7 +411,7 @@ ALAPI void ALAPIENTRY alDistanceModel( ALenum distanceModel );
        * Listener Velocity:        ALfloat[3]
        * Listener Orientation:     ALfloat[6]  (forward and up vector).
        */
-      void                (*alListenerfv)( ALenum pname, ALfloat* param ); 
+      void                (*alListenerfv)( ALenum pname, const ALfloat* param ); 
 
       /*
        * Retrieve listener information.
@@ -406,7 +419,9 @@ ALAPI void ALAPIENTRY alDistanceModel( ALenum distanceModel );
       void              (*alGetListeneri)( ALenum pname, ALint* value );
       void              (*alGetListenerf)( ALenum pname, ALfloat* value );
 
+#ifdef LINUX_AL
       void              (*alGetListeneriv)( ALenum pname, ALint* values );
+#endif
       void              (*alGetListenerfv)( ALenum pname, ALfloat* values );
 
 /**
@@ -422,7 +437,7 @@ ALAPI void ALAPIENTRY alDistanceModel( ALenum distanceModel );
       void                (*alGenSources)( ALsizei n, ALuint* sources ); 
 
       /** Delete Source objects. */
-      void                (*alDeleteSources)( ALsizei n, ALuint* sources );
+      void                (*alDeleteSources)( ALsizei n, const ALuint* sources );
 
       /** Verify a handle is a valid Source. */ 
       ALboolean        (*alIsSource)( ALuint sid ); 
@@ -439,16 +454,17 @@ ALAPI void ALAPIENTRY alDistanceModel( ALenum distanceModel );
 
       /** Set a float vector parameter for a Source object. */
       void                (*alSourcefv)( ALuint sid, ALenum param,
-                              ALfloat* values ); 
+                              const ALfloat* values ); 
 
       /** Get an integer scalar parameter for a Source object. */
       void                (*alGetSourcei)( ALuint sid,
                               ALenum pname, ALint* value );
 
+#ifdef LINUX_AL
       /** Get an integer parameter for a Source object. */
       void                (*alGetSourceiv)( ALuint sid,
                               ALenum pname, ALint* values );
-
+#endif
       /** Get a float scalar parameter for a Source object. */
       void                (*alGetSourcef)( ALuint sid,
                               ALenum pname, ALfloat* value );
@@ -492,10 +508,10 @@ ALAPI void ALAPIENTRY alDistanceModel( ALenum distanceModel );
       /**
        * vector forms of those functions we all love
        */
-      void                (*alSourcePlayv)( ALsizei ns, ALuint *ids );
-      void                (*alSourceStopv)( ALsizei ns, ALuint *ids );
-      void                (*alSourceRewindv)( ALsizei ns, ALuint *ids );
-      void                (*alSourcePausev)( ALsizei ns, ALuint *ids );
+      void                (*alSourcePlayv)( ALsizei ns, const ALuint *ids );
+      void                (*alSourceStopv)( ALsizei ns, const ALuint *ids );
+      void                (*alSourceRewindv)( ALsizei ns, const ALuint *ids );
+      void                (*alSourcePausev)( ALsizei ns, const ALuint *ids );
 
 /**
  * BUFFER
@@ -509,7 +525,7 @@ ALAPI void ALAPIENTRY alDistanceModel( ALenum distanceModel );
 
       /** Buffer object generation. */
       void                (*alGenBuffers)( ALsizei n, ALuint* buffers );
-      void                (*alDeleteBuffers)( ALsizei n, ALuint* buffers );
+      void                (*alDeleteBuffers)( ALsizei n, const ALuint* buffers );
       ALboolean           (*alIsBuffer)( ALuint buffer );
 
       /**
@@ -517,7 +533,7 @@ ALAPI void ALAPIENTRY alDistanceModel( ALenum distanceModel );
        */
       void                (*alBufferData)( ALuint   buffer,
                                         ALenum   format,
-                                        ALvoid*    data,
+                                        const ALvoid*    data,
                                         ALsizei  size,
                                         ALsizei  freq );
 
@@ -525,11 +541,14 @@ ALAPI void ALAPIENTRY alDistanceModel( ALenum distanceModel );
                                       ALenum param, ALint*   value );
       void                (*alGetBufferf)( ALuint buffer,
                                       ALenum param, ALfloat* value );
+#ifdef LINUX_AL
       void                (*alGetBufferiv)( ALuint buffer,
                                       ALenum param, ALint*   value );
       void                (*alGetBufferfv)( ALuint buffer,
                                       ALenum param, ALfloat* value );
+#endif
 
+#ifdef LINUX_AL
 /**
  * EXTENSION: IASIG Level 2 Environment.
  * Environment object generation.
@@ -548,12 +567,17 @@ ALAPI void ALAPIENTRY alDistanceModel( ALenum distanceModel );
                                       ALenum param, ALint value );
       void                (*alEnvironmentfIASIG)( ALuint eid,
                                       ALenum param, ALuint value );
+
+#endif 
+
       /**
        * Queue stuff
        */
+#ifdef LINUX_AL
       void              (*alQueuei)(ALuint sid, ALenum param, ALint value );
+#endif
       void              (*alSourceUnqueueBuffers)(ALuint sid, ALsizei numEntries, ALuint *bids );
-      void              (*alSourceQueueBuffers)(ALuint sid, ALsizei numEntries, ALuint *bids );
+      void              (*alSourceQueueBuffers)(ALuint sid, ALsizei numEntries, const ALuint *bids );
 
       void              (*alDopplerFactor)( ALfloat value );
       void              (*alDopplerVelocity)( ALfloat value );
