@@ -12,7 +12,7 @@
  * defined in software_time_filters or software_frequency_filters
  * is applied to each source that finds its way into the mixer.
  *
- * ApplyFilters takes a chunk of data from the original buffer 
+ * ApplyFilters takes a chunk of data from the original buffer
  * associated with the passed source.
  *
  * This chunk is understood to be that block of data samp->_orig_buffer
@@ -24,7 +24,7 @@
  *
  * Applying filters to a source does not (should not) change the original
  * pcm data.  ApplyFilters will split the original pcm data prior to
- * calling each filter, and filters should restrict themselves to 
+ * calling each filter, and filters should restrict themselves to
  * manipulating the passed data.
  *
  * time domain filters (those defines in software_time_filters) are
@@ -36,14 +36,14 @@
  *	AL_buffer *samp
  *		buffer that the source is associated with
  *	ALshort **buffers
- *		arrays of points to PCM data, one element per 
+ *		arrays of points to PCM data, one element per
  *		channel(left/right/etc)
  *	ALuint nc
  *		number of elements in buffers
  *	ALuint len
  *		byte length of each element in buffers
  *
- * Filters are expected to alter buffers[0..nc-1] in place.  After 
+ * Filters are expected to alter buffers[0..nc-1] in place.  After
  * the ApplyFilter iteration is over, the resulting data is mixed into
  * the main mix buffer and forgotten.  The data altered is cumulative,
  * that is to say, if two filters alf_f and alf_g occur in sequential
@@ -90,7 +90,7 @@
 
 #define USE_TPITCH_LOOKUP 1 /* icculus change here JIV FIXME */
 
-/* 
+/*
  * TPITCH_MAX sets the number of discrete values for AL_PITCH we can have.
  * You can set AL_PITCH to anything, but integer rounding will ensure that
  * it will fall beween MIN_SCALE and 2.0.
@@ -239,12 +239,13 @@ void _alDestroyFilters( void ) {
 		tpitch_lookup.offsets[i] = 0;
 		tpitch_lookup.fractionals[i] = 0;
 	}
+	tpitch_lookup.len = 0;
 #endif
 
 	return;
 }
 
-/* 
+/*
  * _alApplyFilters( ALuint cid, ALuint sid )
  *
  * _alApplyFilters is called from the mixing function, and is passed
@@ -375,7 +376,7 @@ void _alApplyFilters( ALuint cid, ALuint sid ) {
 		}
 	}
 
-	/* 
+	/*
 	 * adjust len to account for end of sample, looping, etc
 	 */
 	if(filterlen > _alSourceBytesLeft(src, samp))
@@ -474,10 +475,10 @@ void _alApplyFilters( ALuint cid, ALuint sid ) {
  *    In BETWEEN, the gain is transitionally set to some value between
  *    what it would be in INSIDE and OUTSIDE.
  *
- * Well, okay that's still pretty goofy.  Folks who want to set a 
+ * Well, okay that's still pretty goofy.  Folks who want to set a
  * minimum attenuation can stil do so using AL_SOURCE_ATTENUATION_MIN.
  *
- * IMPLEMENTATION: 
+ * IMPLEMENTATION:
  * okay, we check the angle between the speaker position and
  * the source's direction vector, using the source's position
  * as the origin.  This angle we call theta.
@@ -628,7 +629,7 @@ void alf_coning( ALuint cid,
 	_alDebug(ALD_SOURCE, __FILE__, __LINE__,
 		"alf_coning: sid %d icone %f ocone %f", src->sid, icone, ocone );
 
-	theta = fabs( _alVectorAngleBeween(sp, lp, srcDir) );
+	theta = _alVectorAngleBetween(sp, lp, srcDir);
 
 	if( theta <= (icone / 2.0f) ) {
 		/*
@@ -660,7 +661,7 @@ void alf_coning( ALuint cid,
 
 		sa += _AL_CUTTOFF_ATTENUATION;
 		sa /= 2;
-	} else { 
+	} else {
 		/*
 		 * OUTSIDE:
 		 *
@@ -691,7 +692,7 @@ void alf_coning( ALuint cid,
  * As far as reverb implementations go, this sucks.  Should be
  * frequency based?
  *
- * Should be able to be applied in sequence for second order 
+ * Should be able to be applied in sequence for second order
  * approximations.
  *
  * FIXME: this is so ugly!  And consumes a ton of memory.
@@ -756,7 +757,7 @@ void alf_reverb( UNUSED(ALuint cid),
 	return;
 }
 
-/* 
+/*
  * alf_da
  *
  * alf_da implements distance attenuation.  We call compute_sa to get the
@@ -807,7 +808,7 @@ void alf_da( ALuint cid,
 	_alcUnlockContext( cid );
 
 	/*
-	 * 
+	 *
 	 * The source specific max is set to max at this point, but may be
 	 * altered below of the application has set it.
 	 */
@@ -1002,7 +1003,7 @@ static void init_tpitch_lookup( ALuint len ) {
  *    It's not a good idea to mess with src's pitch.  Some method of
  *    expressing this computation without changing the source's attributes
  *    should be used.
- * 
+ *
  */
 void alf_tdoppler( ALuint cid,
 		   AL_source *src,
@@ -1020,6 +1021,17 @@ void alf_tdoppler( ALuint cid,
 	AL_sourcestate *srcstate;
 	ALfloat doppler_factor;
 	ALfloat doppler_velocity;
+	ALfloat doppler_pitch;
+
+	/* initialize the mixrate */
+	if(src->pitch.isset == AL_TRUE)
+	{
+		src->mixrate = src->pitch.data;
+	}
+	else
+	{
+		src->mixrate = 1.0;
+	}
 
 	/* lock context, get context specific stuff */
 	_alcLockContext( cid );
@@ -1052,6 +1064,11 @@ void alf_tdoppler( ALuint cid,
 		return;
 	}
 
+        if (fabs(doppler_factor - 1.0E-6) < 0) {
+                /* doppler factor set to zero, no doppler effect */
+                return;
+        }
+
 	if(sv == NULL) {
 		/*
 		 * if unset, set to the velocity to the
@@ -1075,8 +1092,7 @@ void alf_tdoppler( ALuint cid,
 		 *
 		 * FIXME: use epsilon
 		 */
-		src->pitch.data = 1.0;
-		
+
 		return;
 	}
 
@@ -1086,20 +1102,22 @@ void alf_tdoppler( ALuint cid,
 		fprintf(stderr, "weird\n");
 	}
 
-	src->pitch.data = compute_doppler_pitch(lp, lv, sp, sv,
-				doppler_factor, doppler_velocity);
+	doppler_pitch = compute_doppler_pitch(lp, lv, sp, sv,
+					doppler_factor, doppler_velocity);
+
+	src->mixrate *= doppler_pitch;
 
 #ifdef DEBUG
-	if(src->pitch.data < MIN_PITCH)
+	if(src->mixrate < MIN_PITCH)
 	{
 		_alDebug(ALD_FILTER, __FILE__, __LINE__,
-			 "Clamping src->pitch.data %f\n",
-			 src->pitch.data);
+			 "Clamping src->mixrate %f\n",
+			 src->mixrate);
 	}
 #endif
 
-	src->pitch.data = MAX(src->pitch.data, MIN_PITCH);
-	src->pitch.data = MIN(src->pitch.data, 2.0f);
+	src->mixrate = MAX(src->mixrate, MIN_PITCH);
+	src->mixrate = MIN(src->mixrate, 2.0f);
 
 	return;
 }
@@ -1125,13 +1143,13 @@ void alf_minmax( UNUSED(ALuint cid),
 	 * if min or max are set, use them.  Otherwise, keep defaults
 	 */
 	if(aminp != NULL) {
-		attenuation_min = _alDBToLinear(*aminp);
+		attenuation_min = *aminp;
 	} else {
 		_alSourceGetParamDefault( AL_MIN_GAIN, &attenuation_min );
 	}
 
 	if(amaxp != NULL) {
-		attenuation_max = _alDBToLinear(*amaxp);
+		attenuation_max = *amaxp;
 	} else {
 		_alSourceGetParamDefault( AL_MAX_GAIN, &attenuation_max );
 	}
@@ -1169,7 +1187,7 @@ void alf_listenergain( ALuint cid,
 		       "listenergain: got NULL param" );
 		return;
 	}
-	
+
 	gain = * (ALfloat *) temp;
 
 	for(i = 0; i < nc; i++) {
@@ -1200,7 +1218,7 @@ static ALfloat compute_doppler_pitch( ALfloat *object1, ALfloat *o1_vel,
         ALfloat ratio;            /* Ratio of relative velocities */
 	ALfloat retval;           /* final doppler shift */
 
-        /* 
+        /*
          * Set up the "between" vector which points from one object to the
          * other
          */
@@ -1215,7 +1233,7 @@ static ALfloat compute_doppler_pitch( ALfloat *object1, ALfloat *o1_vel,
          * vector.
          *
          * The _alVectorDotp function is not set up for computing dot products
-         * for actual vectors (it works for three points that define two 
+         * for actual vectors (it works for three points that define two
          * vectors from a common origin), so I'll do it here.
          */
         obj1V  = o1_vel[0] * between[0];
@@ -1227,16 +1245,24 @@ static ALfloat compute_doppler_pitch( ALfloat *object1, ALfloat *o1_vel,
         obj2V += o2_vel[1] * between[1];
         obj2V += o2_vel[2] * between[2];
 
-        /* 
+        /*
+         * Apply the Doppler factor by modifying the source and listener
+         * velocities.  This will exaggerate or reduce the Doppler
+         * effect as expected.
+         */
+        obj1V *= factor;
+        obj2V *= factor;
+
+        /*
          * Now compute the obj1/obj2 velocity ratio, taking into account
          * the propagation speed.  This formula is straight from the spec.
          */
-        obj1V = speed + obj1V;
-        obj2V = speed - obj2V;
+        obj1V = speed - obj1V;
+        obj2V = speed + obj2V;
         ratio = obj1V / obj2V;
 
-        /* Finally, scale by the doppler factor */
-	retval = factor * ratio;
+        /* Finally, return the ratio */
+	retval = ratio;
 
 	return retval;
 }
@@ -1247,7 +1273,7 @@ static ALfloat compute_doppler_pitch( ALfloat *object1, ALfloat *o1_vel,
  *
  * this filter acts out AL_PITCH.
  *
- * This filter is implements AL_PITCH, but - oh-ho! - in the 
+ * This filter is implements AL_PITCH, but - oh-ho! - in the
  * time domain.  All that good fft mojo going to waste.
  */
 void alf_tpitch( UNUSED(ALuint cid),
@@ -1264,18 +1290,17 @@ void alf_tpitch( UNUSED(ALuint cid),
 	int *offsets;        /* pointer to set of offsets in lookup table */
 	float *fractionals;  /* pointer to set of fractionals in lookup table */
 	int bufchans;
-	ALfloat *ppitch, pitch;
+	ALfloat pitch;
 
-	ppitch = _alGetSourceParam( src, AL_PITCH );
-	if(ppitch == NULL) {
+	pitch = src->mixrate;
+	
+	if (pitch == 1.0 && !(src->flags & ALS_NEEDPITCH)) {
 		/*
-		 * if pitch is unset, default or invalid, return.
+		 * mixrate is at the default, so changing pitch is unnecessary.
 		 */
 		return;
 	}
 
-	pitch = *ppitch;
-
 	bufchans = _al_ALCHANNELS(samp->format); /* we need bufchans to
 						      * scale our increment
 						      * of the soundpos,
@@ -1416,7 +1441,7 @@ void alf_tpitch( UNUSED(ALuint cid),
 				float firstsample = obufptr[offset];
 				float nextsample = obufptr[nextoffset];
 				int finalsample;
-				
+
 				/* do a little interpolation */
 				finalsample = lrintf(firstsample +
 				            frac * (nextsample - firstsample));
@@ -1432,7 +1457,7 @@ void alf_tpitch( UNUSED(ALuint cid),
 				int firstsample = obufptr[offset];
 				int nextsample = obufptr[nextoffset];
 				int finalsample;
-				
+
 				/* do a little interpolation */
 				finalsample = firstsample +
 				            frac * (nextsample - firstsample);
@@ -1460,7 +1485,7 @@ void alf_tpitch( UNUSED(ALuint cid),
 	 */
 	ipos = (int) (len * pitch);
 	src->srcParams.soundpos += bufchans * ipos * sizeof(ALshort);
-	
+
 	if(src->srcParams.soundpos > samp->size)
 	{
 		/*
@@ -1468,13 +1493,13 @@ void alf_tpitch( UNUSED(ALuint cid),
 		 *
 		 * Since we're handling the soundpos incrementing for
 		 * this source (usually done in _alMixSources), we have
-		 * to handle all the special cases here instead of 
+		 * to handle all the special cases here instead of
 		 * delegating them.
 		 *
-		 * These include callback, looping, and streaming 
+		 * These include callback, looping, and streaming
 		 * sources.  For now, we just handle looping and
-		 * normal sources, as callback sources will probably 
-		 * require added some special case logic to _alSplitSources 
+		 * normal sources, as callback sources will probably
+		 * require added some special case logic to _alSplitSources
 		 * to give up a little more breathing room.
 		 */
 		if( _alSourceIsLooping( src ) == AL_TRUE ) {
@@ -1508,7 +1533,7 @@ void alf_tpitch( UNUSED(ALuint cid),
  *
  * this filter acts out AL_PITCH.
  *
- * This filter is implements AL_PITCH, but - oh-ho! - in the 
+ * This filter is implements AL_PITCH, but - oh-ho! - in the
  * time domain.  All that good fft mojo going to waste.
  */
 void alf_tpitch( UNUSED(ALuint cid),
@@ -1524,18 +1549,17 @@ void alf_tpitch( UNUSED(ALuint cid),
 	ALuint i;
 	ALuint clen;
 	int bufchans;
-	ALfloat *ppitch, pitch;
+	ALfloat pitch;
 
-	ppitch = _alGetSourceParam( src, AL_PITCH );
-	if(ppitch == NULL) {
+	pitch = src->mixrate;
+	
+	if (pitch == 1.0 && !(src->flags & ALS_NEEDPITCH)) {
 		/*
-		 * if pitch is unset, default or invalid, return.
+		 * mixrate is at the default, so changing pitch is unnecessary.
 		 */
 		return;
 	}
 
-	pitch = *ppitch;
-
 	bufchans = _al_ALCHANNELS(samp->format); /* we need bufchans to
 						      * scale our increment
 						      * of the soundpos,
@@ -1547,8 +1571,8 @@ void alf_tpitch( UNUSED(ALuint cid),
 	 * if pitch is out of range, clamp.
 	 */
 	pitch = MIN(pitch, 2.0f);
-	pitch = MAX(pitch, MIN_PITCH);	
-	
+	pitch = MAX(pitch, MIN_PITCH);
+
 	/*
 	 *  We need len in samples, not bytes.
 	 */
@@ -1618,7 +1642,7 @@ void alf_tpitch( UNUSED(ALuint cid),
 				int firstsample = obufptr[(int) (j * pitch)];
 				int nextsample = obufptr[(int)((j+1) * pitch)];
 				int finalsample;
-				
+
 				/* do a little interpolation */
 				finalsample = firstsample +
 				            frac * (nextsample - firstsample);
@@ -1643,7 +1667,7 @@ void alf_tpitch( UNUSED(ALuint cid),
 	 */
 	ipos = (int) (len * pitch);
 	src->srcParams.soundpos += bufchans * ipos * sizeof(ALshort);
-	
+
 	if(src->srcParams.soundpos > samp->size)
 	{
 		/*
@@ -1651,13 +1675,13 @@ void alf_tpitch( UNUSED(ALuint cid),
 		 *
 		 * Since we're handling the soundpos incrementing for
 		 * this source (usually done in _alMixSources), we have
-		 * to handle all the special cases here instead of 
+		 * to handle all the special cases here instead of
 		 * delegating them.
 		 *
-		 * These include callback, looping, and streaming 
+		 * These include callback, looping, and streaming
 		 * sources.  For now, we just handle looping and
-		 * normal sources, as callback sources will probably 
-		 * require added some special case logic to _alSplitSources 
+		 * normal sources, as callback sources will probably
+		 * require added some special case logic to _alSplitSources
 		 * to give up a little more breathing room.
 		 */
 		if( _alSourceIsLooping( src ) == AL_TRUE ) {
@@ -1743,16 +1767,16 @@ static ALfloat compute_sa( ALfloat *source_pos, ALfloat source_max,
 
 	if( retval > 1.0 ) {
 		return 1.0;
-	} 
+	}
 
 	if(retval < _AL_CUTTOFF_ATTENUATION) {
 		return _AL_CUTTOFF_ATTENUATION;
 	}
-	
+
 	return retval;
 }
 
-/* 
+/*
  * alf_panning
  *
  */
@@ -1789,5 +1813,5 @@ void alf_panning( ALuint cid,
 		sa += 1.0;
 
 		src->srcParams.gain[i] *= sa;
-	}	
+	}
 }
