@@ -56,7 +56,7 @@
 /*
  * mixer thread's ID, if it needs one
  */
-extern ThreadID mixthread; 
+extern ThreadID mixthread;
 
 /*
  * pcm buffers that filters act on
@@ -127,6 +127,10 @@ ALboolean _alInit( void ) {
 	_alRegisterExtensionGroup( (const ALubyte*) "AL_LOKI_play_position" );
 	_alRegisterExtensionGroup( (const ALubyte*) "AL_LOKI_quadriphonic" );
 
+#ifdef SMPEG_SUPPORT
+	 _alRegisterExtensionGroup( (const ALubyte*) "AL_EXT_MP3" );
+#endif /* SMPEG_SUPPORT */
+
 #ifdef VORBIS_SUPPORT
 	_alRegisterExtensionGroup( (const ALubyte*) "AL_EXT_vorbis" );
 #endif /* CAPTURE_SUPPORT */
@@ -165,7 +169,7 @@ void _alExit( void ) {
 	/* we could be sync, so we check mixthread for a valid ID */
 	if(mixthread != NULL) {
 		time_for_mixer_to_die = AL_TRUE;
-		
+
 		tlWaitThread( mixthread );
 
 		while( time_for_mixer_to_die == AL_TRUE ) {
@@ -213,7 +217,7 @@ void _alExit( void ) {
 #ifdef DEBUG_STUB
 void _alStub( const char *str ) {
 	fprintf(stderr, "%s stub function\n", str);
-	
+
 	return;
 }
 #endif
@@ -426,7 +430,8 @@ void _alBuffersCopy(void **dsts, void **srcs, int len, int nc) {
 }
 
 /*
- * _alRotatePointAboutAxis( ALfloat angle, ALfloat *point, ALfloat *axis )
+ * _alRotatePointAboutAxis( const ALfloat angle, ALfloat *point,
+ *                          const ALfloat *axis )
  *
  * Rotates point angle radians about axis.
  *
@@ -437,7 +442,8 @@ void _alBuffersCopy(void **dsts, void **srcs, int len, int nc) {
  * FIXME: check my math
  * FIXME: needs to check args
  */
-void _alRotatePointAboutAxis( ALfloat angle, ALfloat *point, ALfloat *axis ) {
+void _alRotatePointAboutAxis( const ALfloat angle, ALfloat *point,
+                              const ALfloat *axis ) {
 	ALmatrix *m;
 	ALmatrix *pm;
 	ALmatrix *rm;
@@ -503,7 +509,7 @@ void _alRotatePointAboutAxis( ALfloat angle, ALfloat *point, ALfloat *axis ) {
 	for(i = 0; i < 3; i++) {
 		point[i] = rm->data[0][i];
 	}
-	
+
 	_alMatrixFree(m);
 	_alMatrixFree(pm);
 	_alMatrixFree(rm);
@@ -602,7 +608,7 @@ void _alMatrixFree(ALmatrix *m) {
  * slurp file named by fname to into *buffer, mallocing memory.
  */
 int _alSlurp(const char *fname, void **buffer) {
-	struct stat buf;	
+	struct stat buf;
 	FILE *fh;
 	ALint len;
 
@@ -800,104 +806,6 @@ ALboolean _alIsZeroVector(ALfloat *fv)
 	return AL_TRUE;
 }
 
-/*
- * _alLinearToDB( ALfloat linear )
- *
- * Convert a linear gain to a logarithmic one.
- */
-ALfloat _alLinearToDB(ALfloat linear)
-{
-	static const float logtab[] = {
-		0.00, 0.001, 0.002, 0.003, 0.004, 0.005, 0.01, 0.011,
-		0.012, 0.013, 0.014, 0.015, 0.016, 0.02, 0.021, 0.022,
-		0.023, 0.024, 0.025, 0.03, 0.031, 0.032, 0.033, 0.034,
-		0.04, 0.041, 0.042, 0.043, 0.044, 0.05, 0.051, 0.052,
-		0.053, 0.054, 0.06, 0.061, 0.062, 0.063, 0.064, 0.07,
-		0.071, 0.072, 0.073, 0.08, 0.081, 0.082, 0.083, 0.084,
-		0.09, 0.091, 0.092, 0.093, 0.094, 0.10, 0.101, 0.102,
-		0.103, 0.11, 0.111, 0.112, 0.113, 0.12, 0.121, 0.122,
-		0.123, 0.124, 0.13, 0.131, 0.132, 0.14, 0.141, 0.142,
-		0.143, 0.15, 0.151, 0.152, 0.16, 0.161, 0.162, 0.17,
-		0.171, 0.172, 0.18, 0.181, 0.19, 0.191, 0.192, 0.20,
-		0.201, 0.21, 0.211, 0.22, 0.221, 0.23, 0.231, 0.24, 0.25,
-		0.251, 0.26, 0.27, 0.271, 0.28, 0.29, 0.30, 0.301, 0.31,
-		0.32, 0.33, 0.34, 0.35, 0.36, 0.37, 0.38, 0.39, 0.40,
-		0.41, 0.43, 0.50, 0.60, 0.65, 0.70, 0.75, 0.80, 0.85,
-		0.90, 0.95, 0.97, 0.99 };
-	const int logmax = sizeof logtab / sizeof *logtab;
-
-	if(linear <= 0.0) {
-		return 0.0;
-	}
-
-	if(linear >= 1.0) {
-		return 1.0;
-	}
-
-	return logtab[(int) (logmax * linear)];
-}
-
-/*
- * _alDBToLinear( ALfloat dBs )
- *
- * Convert a logarithmic gain to a linear one.
- *
- * FIXME: So kludgey.
- */
-ALfloat _alDBToLinear(ALfloat dBs) {
-	static const float logtab[] = {
-		0.00, 0.001, 0.002, 0.003, 0.004, 0.005, 0.01, 0.011,
-		0.012, 0.013, 0.014, 0.015, 0.016, 0.02, 0.021, 0.022,
-		0.023, 0.024, 0.025, 0.03, 0.031, 0.032, 0.033, 0.034,
-		0.04, 0.041, 0.042, 0.043, 0.044, 0.05, 0.051, 0.052,
-		0.053, 0.054, 0.06, 0.061, 0.062, 0.063, 0.064, 0.07,
-		0.071, 0.072, 0.073, 0.08, 0.081, 0.082, 0.083, 0.084,
-		0.09, 0.091, 0.092, 0.093, 0.094, 0.10, 0.101, 0.102,
-		0.103, 0.11, 0.111, 0.112, 0.113, 0.12, 0.121, 0.122,
-		0.123, 0.124, 0.13, 0.131, 0.132, 0.14, 0.141, 0.142,
-		0.143, 0.15, 0.151, 0.152, 0.16, 0.161, 0.162, 0.17,
-		0.171, 0.172, 0.18, 0.181, 0.19, 0.191, 0.192, 0.20,
-		0.201, 0.21, 0.211, 0.22, 0.221, 0.23, 0.231, 0.24, 0.25,
-		0.251, 0.26, 0.27, 0.271, 0.28, 0.29, 0.30, 0.301, 0.31,
-		0.32, 0.33, 0.34, 0.35, 0.36, 0.37, 0.38, 0.39, 0.40,
-		0.41, 0.43, 0.50, 0.60, 0.65, 0.70, 0.75, 0.80, 0.85,
-		0.90, 0.95, 0.97, 0.99 };
-	const int logmax = sizeof logtab / sizeof *logtab;
-	int max = logmax;
-	int min = 0;
-	int mid;
-	int last = -1;
-
-	if(dBs <= 0.0) {
-		return 0.0;
-	}
-
-	if(dBs >= 1.0) {
-		return 1.0;
-	}
-
-	mid = (max - min) / 2;
-	do {
-		last = mid;
-
-		if(logtab[mid] == dBs) {
-			break;
-		}
-
-		if(logtab[mid] < dBs) {
-			/* too low */
-			min = mid;
-		} else { 
-			/* too high */
-			max = mid;
-		}
-
-		mid = min + ((max - min) / 2);
-	} while(last != mid);
-
-	return ((float) mid / logmax);
-}
-
 /*
  *  _al_RAWFORMAT( ALenum format )
  *
