@@ -73,7 +73,7 @@ static ALshort *stereoptr = NULL; /*
 				   * scratch space for splitting multichannels
 				   * sources.
 				   */
-				   
+
 /*
  * special split source to handle callbacks
  */
@@ -110,7 +110,7 @@ void *_alSourceGetBufptr(AL_source *src, AL_buffer *buf, ALuint ind);
 /*
  * alIsSource( ALuint sid )
  *
- * Returns AL_TRUE if sid is a currently valid source id, 
+ * Returns AL_TRUE if sid is a currently valid source id,
  * AL_FALSE otherwise.
  */
 ALboolean alIsSource( ALuint sid )
@@ -129,7 +129,7 @@ ALboolean alIsSource( ALuint sid )
 /*
  * _alIsSource( ALuint sid )
  *
- * Returns AL_TRUE if sid is a currently valid source id, 
+ * Returns AL_TRUE if sid is a currently valid source id,
  * AL_FALSE otherwise.
  *
  * Assumes locked source
@@ -208,7 +208,7 @@ void alGenSources( ALsizei n, ALuint *buffer ) {
 		if( sindex == -1 ) {
 			/* We ran into a problem somewhere in
 			 * allocing a set of source ids.  Run
-			 * through the ones we did alloc, 
+			 * through the ones we did alloc,
 			 * realloc them, set error and return.
 			 *
 			 * FIXME: Should unlock, or have a non-locking
@@ -260,13 +260,13 @@ void alGenSources( ALsizei n, ALuint *buffer ) {
 void _alSource2D( AL_source *src ) {
 	_alDebug(ALD_MAXIMUS, __FILE__, __LINE__,
 			"_alSource2D: source turned 2D");
-		
+
 	src->position.isset = AL_FALSE;
 	_alSourceGetParamDefault( AL_POSITION, src->position.data );
 
 	src->direction.isset = AL_FALSE;
 	_alSourceGetParamDefault( AL_DIRECTION, src->direction.data );
-	
+
 	src->coneinnerangle.isset = AL_FALSE;
 	_alSourceGetParamDefault( AL_CONE_INNER_ANGLE, &src->coneinnerangle.data );
 
@@ -350,9 +350,9 @@ void alSourcei( ALuint sid, ALenum param, ALint i1 )
 		 */
 		_alDebug(ALD_SOURCE, __FILE__, __LINE__,
 			"alSourcei: source id %d is not valid", sid);
-		
+
 		_alDCSetError( AL_INVALID_NAME );
-		
+
 		SOURCEUNLOCK();
 		return;
 	}
@@ -391,7 +391,7 @@ void alSourcei( ALuint sid, ALenum param, ALint i1 )
 		      sid, param, i1);
 
 		_alDCSetError( AL_INVALID_VALUE );
-		
+
 		SOURCEUNLOCK();
 		return;
 	}
@@ -444,7 +444,7 @@ void alSourcei( ALuint sid, ALenum param, ALint i1 )
 			_alDebug(ALD_SOURCE, __FILE__, __LINE__,
 				"alSourcei: invalid or stubbed source param 0x%x",
 				param );
-			
+
 			_alDCSetError( AL_ILLEGAL_ENUM );
 			break;
 	}
@@ -545,7 +545,7 @@ void alSource3f( ALuint sid, ALenum param,
  * If any member of fv is out of range for the attribute, AL_INVALID_VALUE.
  *
  */
-void alSourcefv( ALuint sid, ALenum param, ALfloat *fv1 )
+void alSourcefv( ALuint sid, ALenum param, const ALfloat *fv1 )
 {
 	AL_source *source;
 	ALboolean inrange = AL_TRUE;
@@ -585,8 +585,8 @@ void alSourcefv( ALuint sid, ALenum param, ALfloat *fv1 )
 			return;
 			break;
 	}
-	
-	
+
+
 	SOURCELOCK();
 	source = _alDCGetSource( sid );
 
@@ -619,7 +619,7 @@ void alSourcefv( ALuint sid, ALenum param, ALfloat *fv1 )
 		case AL_CONE_OUTER_GAIN:
 		case AL_GAIN:
 		case AL_GAIN_LINEAR_LOKI:
-		  inrange = _alCheckRangef( fv1[0], 0.0, 1.0 );
+		  inrange = _alCheckRangef( fv1[0], 0.0, FLT_MAX );
 		  break;
 		case AL_CONE_INNER_ANGLE:
 		case AL_CONE_OUTER_ANGLE:
@@ -658,7 +658,7 @@ void alSourcefv( ALuint sid, ALenum param, ALfloat *fv1 )
 		_alDCSetError( AL_INVALID_VALUE );
 
 		SOURCEUNLOCK();
-		
+
 		return;
 	}
 
@@ -700,7 +700,6 @@ void alSourcefv( ALuint sid, ALenum param, ALfloat *fv1 )
 		   *
 		   */
 		  source->flags      |= ALS_NEEDPITCH;
-		  source->pitch.isset = AL_TRUE;
 		  break;
 		case AL_MIN_GAIN:
 		  source->attenuationmin.isset = AL_TRUE;
@@ -728,7 +727,13 @@ void alSourcefv( ALuint sid, ALenum param, ALfloat *fv1 )
 			source->pitch.isset = AL_FALSE;
 			source->pitch.data  = 1.0;
 
-			source->flags &= ~ALS_NEEDPITCH;
+			/*
+			 * If velocity is set, then doppler may 
+			 * change the pitch.
+			 */
+			if (!source->velocity.isset) {
+				source->flags &= ~ALS_NEEDPITCH;
+			}
 		  } else {
 		  	source->pitch.isset = AL_TRUE;
 		  	source->pitch.data  = fv1[0];
@@ -738,9 +743,6 @@ void alSourcefv( ALuint sid, ALenum param, ALfloat *fv1 )
 		  }
 		  break;
 		case AL_GAIN:
-		  source->gain.isset = AL_TRUE;
-		  source->gain.data = _alDBToLinear( fv1[0] );
-		  break;
 		case AL_GAIN_LINEAR_LOKI:
 		  source->gain.isset = AL_TRUE;
 		  source->gain.data = fv1[0];
@@ -760,7 +762,7 @@ void alSourcefv( ALuint sid, ALenum param, ALfloat *fv1 )
 		default:
 		  _alDebug(ALD_SOURCE, __FILE__, __LINE__,
 			  "alSourcefv(%d): param 0x%x not valid", sid, param );
-		  
+
 		  _alDCSetError( AL_ILLEGAL_ENUM );
 		  break;
 	}
@@ -885,7 +887,7 @@ void alGetSourceiv( ALuint sid, ALenum param, ALint *retref )
 
 	/*
 	 * get param value, and store it in temp.  We need it
-	 * more most, but not all, of the following params enums.
+	 * for most, but not all, of the following params enums.
 	 */
 	temp = _alGetSourceParam( src, param );
 	if(temp != NULL) {
@@ -1036,10 +1038,10 @@ void alGetSource3f( ALuint sid, ALenum param,
 		/* silently ignore */
 		_alDebug( ALD_SOURCE, __FILE__, __LINE__,
 			"alGetSource3f: value passed is NULL" );
-		
+
 		return;
 	}
-	
+
 	alGetSourcefv( sid, param, safety_first );
 
 	if(value1)
@@ -1077,7 +1079,7 @@ void alGetSourcef( ALuint sid, ALenum param, ALfloat *value )
 		/* silently ignore */
 		_alDebug( ALD_SOURCE, __FILE__, __LINE__,
 			"alGetSourcef: value passed is NULL" );
-		
+
 		return;
 	}
 
@@ -1159,7 +1161,7 @@ void alGetSourcefv( ALuint sid, ALenum param, ALfloat *values ) {
 	if(src == NULL) {
 		_alDebug(ALD_SOURCE, __FILE__, __LINE__,
 			"alGetSourcefv: source id %d is invalid", sid);
-		
+
 		_alDCSetError( AL_INVALID_NAME );
 
 		SOURCEUNLOCK();
@@ -1171,7 +1173,7 @@ void alGetSourcefv( ALuint sid, ALenum param, ALfloat *values ) {
 
 		_alDebug( ALD_SOURCE, __FILE__, __LINE__,
 			"alGetSourcefv: values passed is NULL", values );
-		
+
 		SOURCEUNLOCK();
 
 		return;
@@ -1198,27 +1200,6 @@ void alGetSourcefv( ALuint sid, ALenum param, ALfloat *values ) {
 	switch( param ) {
 		/* scalars */
 		case AL_GAIN:
-			/*
-			 * AL_GAIN reflects the dB of AL_GAIN_LINEAR_LOKI, so
-			 * we compute this from AL_GAIN_LINEAR_LOKI.
-			 */
-			srcvals = _alGetSourceParam( src, AL_GAIN_LINEAR_LOKI );
-			if( srcvals != NULL) {
-			/*
-			 * If srcval is not NULL, the param in question has
-			 * been explicitly set, which means that we don't
-			 * need to set it to defaults below.
-			 */
-			values[0] = _alLinearToDB( *srcvals );
-
-			SOURCEUNLOCK();
-			return;
-			}
-
-			/* AL_GAIN_LINEAR_LOKI was not set, so we set default */
-			_alSourceGetParamDefault( AL_GAIN_LINEAR_LOKI, values );
-			values[0] = _alLinearToDB( values[0] );
-			break;
 		case AL_MIN_GAIN:
 		case AL_MAX_GAIN:
 		case AL_GAIN_LINEAR_LOKI:
@@ -1252,15 +1233,15 @@ void alGetSourcefv( ALuint sid, ALenum param, ALfloat *values ) {
 /*
  * _alGetSource( ALuint cid, ALuint sid )
  *
- * Returns the address of the source sid from the 
- * context cid, or NULL if the cid or sid is 
+ * Returns the address of the source sid from the
+ * context cid, or NULL if the cid or sid is
  * invalid.
  *
  * Assumes locked context
  */
 AL_source *_alGetSource( ALuint cid, ALuint sid ) {
 	AL_context *cc;
-	
+
 	cc  = _alcGetContext( cid );
 	if(cc == NULL) {
 		/*
@@ -1311,7 +1292,7 @@ void _alSplitSources( ALuint cid,
 	 * the first initialization and we need to allocate the
 	 * stereoptr (stereoptr being that buffer where we mix this
 	 * sort of thing).
-	 * 
+	 *
 	 * or buflen is equal to len and stereoptr is null because
 	 * the device was destroyed and then restarted.
 	 */
@@ -1397,7 +1378,7 @@ void _alSplitSources( ALuint cid,
 			return;
 		}
 	}
-	
+
 	if(_alSourceGetPendingBids(src) > 0)
 		assert(src->bid_queue.read_index < src->bid_queue.size );
 
@@ -1479,7 +1460,7 @@ static void _alSplitSourceCallback( ALuint cid,
 		/* FIXME:
 		 *
 		 * offset memset at resultsamps / width to end
-		 
+
 		 memset(stereoptr, 0, len);
 		 */
 
@@ -1586,7 +1567,7 @@ static void _alSplitSourceLooping( ALuint cid,
 			/* copy samp again and again */
 			for(i = mixable; i < (signed int) len; i += samp->size) {
 				int copylen;
-				
+
 				if(i + samp->size < len) {
 					copylen = samp->size;
 				} else {
@@ -1598,7 +1579,7 @@ static void _alSplitSourceLooping( ALuint cid,
 
 			for(i = 0; i < mixable; i += samp->size) {
 				int copylen;
-				
+
 				if(i + samp->size < (unsigned int) mixable) {
 					copylen = samp->size;
 				} else {
@@ -1683,9 +1664,9 @@ static void _alSplitSourceQueue( ALuint cid,
 #if 1
 	if( _alSourceBytesLeftByChannel(src, samp) <= 0)
 	{
-		
+
 		ALuint bid = 0;
-		
+
 		/* no data left in this buffer */
 		src->bid_queue.read_index++;
 		if(src->bid_queue.read_index >= src->bid_queue.size)
@@ -1705,6 +1686,13 @@ static void _alSplitSourceQueue( ALuint cid,
 
 		bid  = src->bid_queue.queue[src->bid_queue.read_index];
 		samp = _alGetBuffer(bid);
+		if(samp == NULL)
+		{
+			_alDebug(ALD_SOURCE, __FILE__, __LINE__,
+				 "_alSplitSourceQueue: null buffer");
+
+			return;
+		}
 
 		src->srcParams.soundpos = 0;
 
@@ -1762,7 +1750,7 @@ static void _alSplitSourceQueue( ALuint cid,
 			 "first buffer %d, second buffer %d",
 			 mixable, remaining);
 #endif
-		
+
 
 		for(i = 0; i < nc; i++) {
 			bufptr = _alSourceGetBufptr(src, samp, i);
@@ -1773,7 +1761,7 @@ static void _alSplitSourceQueue( ALuint cid,
 		}
 
 		src->srcParams.new_readindex = src->bid_queue.read_index + 1;
-		src->srcParams.new_soundpos = nextsamp->size - remaining;
+		src->srcParams.new_soundpos = remaining;
 
 		if(remaining >= nextsamp->size)
 		{
@@ -1795,7 +1783,7 @@ static void _alSplitSourceQueue( ALuint cid,
 		len,
 		samp->size - src->srcParams.soundpos);
 #endif
-			
+
 	/*
 	 * We need to get data from more than one buffer
 	 */
@@ -1805,7 +1793,9 @@ static void _alSplitSourceQueue( ALuint cid,
 
 		if( src->bid_queue.read_index >= src->bid_queue.size )
 		{
+#if DEBUG_QUEUE
 			fprintf(stderr, "end of buffer queue\n" );
+#endif
 
 			/*
 			 * Read past the last buffer and we still
@@ -1818,7 +1808,7 @@ static void _alSplitSourceQueue( ALuint cid,
 				       0, len - collected_bytes);
 			}
 
-			src->srcParams.new_readindex=src->bid_queue.read_index;
+			src->srcParams.new_readindex= 0;
 			src->srcParams.new_soundpos = 0;
 
 			src->bid_queue.read_index = old_readindex;
@@ -1834,14 +1824,15 @@ static void _alSplitSourceQueue( ALuint cid,
 
 		mixable = samp->size - src->srcParams.soundpos;
 
-		if(mixable > (int) len)
+		if(mixable > (int) (len - collected_bytes))
 		{
-			mixable = len;
-			new_soundpos = mixable;
+			mixable = len - collected_bytes;
+			new_soundpos = src->srcParams.soundpos + mixable;
 		}
 		else
 		{
 			src->bid_queue.read_index++;
+			new_soundpos = 0;
 		}
 
 		/* copy from current buffer */
@@ -1854,8 +1845,7 @@ static void _alSplitSourceQueue( ALuint cid,
 
 		collected_bytes += mixable;
 
-		src->srcParams.soundpos = 0;
-		new_soundpos = 0;
+		src->srcParams.soundpos = new_soundpos;
 	}
 
 	src->srcParams.new_readindex = src->bid_queue.read_index;
@@ -1874,7 +1864,7 @@ static void _alSplitSourceQueue( ALuint cid,
  *
  * Copies the interleaved data from src[0..(src_size/2)-1] to
  * dstret[0..dest_channels-1], splitting it into seperate channels.
- * src_channels describes the period of the channel repitition in src,
+ * src_channels describes the period of the channel repetition in src,
  * dest_channels describes the number of independant buffers in dstref.
  * src_size is the size of src in bytes, and offset is the offset into each
  * seperate channel in dstref where the copying begins.
@@ -1942,7 +1932,7 @@ static void _alMonoifyOffset1to4( ALshort **dsts, ALuint offset,
 	ALshort *dst1 = dsts[1];
 	ALshort *dst2 = dsts[2];
 	ALshort *dst3 = dsts[3];
-	
+
 	int len      = size / sizeof *src;
         int i;
 
@@ -1973,7 +1963,7 @@ static void _alMonoifyOffset2to4( ALshort **dsts, ALuint offset,
 	ALshort *dst1 = dsts[1];
 	ALshort *dst2 = dsts[2];
 	ALshort *dst3 = dsts[3];
-	
+
 	int len      = size / sizeof *src;
         int i;
 
@@ -2004,7 +1994,7 @@ static void _alMonoifyOffset4to4( ALshort **dsts, ALuint offset,
 	ALshort *dst1 = dsts[1];
 	ALshort *dst2 = dsts[2];
 	ALshort *dst3 = dsts[3];
-	
+
 	int len      = size / sizeof *src;
         int i;
 
@@ -2147,12 +2137,12 @@ void _alChannelify2Offset( ALshort *dst, ALuint offset,
 }
 
 /*
- *  _alChannelify4Offset( ALshort *dst, ALuint offset,  
- *                        ALshort **srcs, ALuint size ) 
+ *  _alChannelify4Offset( ALshort *dst, ALuint offset,
+ *                        ALshort **srcs, ALuint size )
  *
  * This function is like ChannelifyOffset, but specificly for those cases
  * where the number of channels in srcs is 4.
- *              
+ *
  * assumes locked context
  */
 void _alChannelify4Offset( ALshort *dst, ALuint offset,
@@ -2162,7 +2152,7 @@ void _alChannelify4Offset( ALshort *dst, ALuint offset,
 	ALshort *src2 = &srcs[2][offset / sizeof *srcs];
 	ALshort *src3 = &srcs[3][offset / sizeof *srcs];
         ALuint k;
-        
+
         size /= sizeof *dst; /* we need sample offsets */
 
         for( k = 0; k < size; k++ ) {
@@ -2170,12 +2160,12 @@ void _alChannelify4Offset( ALshort *dst, ALuint offset,
                 dst[1] = src1[k];
 		dst[2] = src2[k];
 		dst[3] = src3[k];
-  
+
                 dst += 4;
         }
-  
+
         return;
-}  
+}
 
 
 
@@ -2190,7 +2180,7 @@ void _alChannelify4Offset( ALshort *dst, ALuint offset,
  * If n is 0, this is a legal nop.  If n < 0, INVALID_VALUE is set
  * and this is a nop.
  */
-void alDeleteSources( ALsizei n, ALuint *sources ) {
+void alDeleteSources( ALsizei n, const ALuint *sources ) {
 	AL_source *src;
 	AL_context *cc;
 	int i;
@@ -2213,7 +2203,7 @@ void alDeleteSources( ALsizei n, ALuint *sources ) {
 	cc = _alcDCGetContext();
         if(cc == NULL) {
 		/*
-		 * No current context with which to evaluate the 
+		 * No current context with which to evaluate the
 		 * validity of the sources
 		 */
 		_alcDCUnlockContext();
@@ -2244,7 +2234,7 @@ void alDeleteSources( ALsizei n, ALuint *sources ) {
 		}
 
 		if((src->state == AL_PLAYING) || (src->state == AL_PAUSED)) {
-			/* 
+			/*
 			 * FIXME: illegal to delete playing source?
 			 */
 			_alDebug(ALD_SOURCE, __FILE__, __LINE__,
@@ -2270,7 +2260,7 @@ void alDeleteSources( ALsizei n, ALuint *sources ) {
 		}
 
 		if(src->state == AL_PLAYING) {
-			/* 
+			/*
 			 * FIXME: illegal to delete playing source?
 			 */
 			_alDebug(ALD_SOURCE, __FILE__, __LINE__,
@@ -2433,7 +2423,7 @@ void alSourceRewind( ALuint sid ) {
  * If any member of sids[0..ns-1] is not a valid source name, set
  * AL_INVALID_NAME.
  */
-void alSourceRewindv( ALsizei ns, ALuint *sids ) {
+void alSourceRewindv( ALsizei ns, const ALuint *sids ) {
 	AL_source *src;
 	ALsizei i;
 
@@ -2509,10 +2499,10 @@ void alSourceRewindv( ALsizei ns, ALuint *sids ) {
  * If ns < 0, sets AL_INVALID_VALUE.
  *
  * If sids[0..ns-1] are all valid source names, then for each source with
- * state AL_PLAYING or AL_PAUSED change the state to AL_STOPPED.  Otherwise, 
+ * state AL_PLAYING or AL_PAUSED change the state to AL_STOPPED.  Otherwise,
  * set AL_INVALID_NAME.
  */
-void alSourceStopv( ALsizei ns, ALuint *sids ) {
+void alSourceStopv( ALsizei ns, const ALuint *sids ) {
 	ALsizei i;
 
 	if( ns == 0 ) {
@@ -2565,7 +2555,7 @@ void alSourceStopv( ALsizei ns, ALuint *sids ) {
  * state AL_INITIAL, AL_STOPPED or AL_PAUSED, change the state to AL_PLAYING.
  * Otherwise, set AL_INVALID_NAME.
  */
-void alSourcePlayv( ALsizei ns, ALuint *sids ) {
+void alSourcePlayv( ALsizei ns, const ALuint *sids ) {
 	ALsizei i;
 
 	if( ns == 0 ) {
@@ -2619,7 +2609,7 @@ void alSourcePlayv( ALsizei ns, ALuint *sids ) {
  * AL_INVALID_NAME.
  *
  */
-void alSourcePausev( ALsizei ns, ALuint *sids ) {
+void alSourcePausev( ALsizei ns, const ALuint *sids ) {
 	AL_source *src;
 	ALsizei i;
 
@@ -2750,7 +2740,7 @@ static void _alInitSource( ALuint sid ) {
 	AL_source *src;
 	AL_sourcestate *srcstate;
 	int i;
-	
+
 	src = _alDCGetSource( sid );
 	if(src == NULL) {
 		/* sid is invalid */
@@ -2791,7 +2781,7 @@ static void _alInitSource( ALuint sid ) {
 
 	src->direction.isset = AL_FALSE;
 	_alSourceGetParamDefault( AL_DIRECTION, src->direction.data );
-	
+
 	src->velocity.isset = AL_FALSE;
 	_alSourceGetParamDefault( AL_VELOCITY, src->velocity.data );
 
@@ -2801,6 +2791,11 @@ static void _alInitSource( ALuint sid ) {
 	src->reverb_scale = 0.25;
 	src->reverb_delay = 0.00;
 
+	/*
+	 * initialize default mix rate
+	 */
+	src->mixrate = 1.0;
+	
 	/*
 	 * initialize gain
 	 */
@@ -2832,7 +2827,7 @@ static void _alInitSource( ALuint sid ) {
 
 	src->isstreaming.isset = AL_FALSE;
 	_alSourceGetParamDefault( AL_STREAMING, &src->isstreaming.data );
- 
+
 	src->isrelative.isset = AL_FALSE;
 	_alSourceGetParamDefault( AL_SOURCE_RELATIVE, &src->isrelative.data );
 
@@ -2939,7 +2934,7 @@ void _alSourceParamApply( AL_source *src,
 	ALuint sampLen;
 	ALuint i;
 	ALfloat gain;
- 
+
 	sampLen = len / sizeof(ALshort); /* we pass sample length */
 
 	for(i = 0; i < nc; i++) {
@@ -2986,9 +2981,8 @@ ALboolean _alSourceShouldIncrement(AL_source *src) {
 	AL_sourcestate *srcstate;
 
 	srcstate =_alSourceQueueGetCurrentState(src);
-	assert(srcstate);
 
-	if(src->flags & ALS_NEEDPITCH) {
+	if((srcstate == NULL) || (src->flags & ALS_NEEDPITCH)) {
 		return AL_FALSE;
 	}
 
@@ -3049,7 +3043,7 @@ ALint _alSourceBytesLeftByChannel(AL_source *src, AL_buffer *samp) {
 
 /*
  *
- * _alSourceIsQueue( AL_source * src ) 
+ * _alSourceIsQueue( AL_source * src )
  *
  * Returns AL_TRUE if source (src) has its queue set to AL_TRUE, AL_FALSE otherwise.
  */
@@ -3099,7 +3093,7 @@ ALboolean FL_alLockSource( UNUSED(const char *fn),
 	}
 
 	_alLockPrintf("_alLockSource", fn, ln);
-	
+
 	sindex = spool_sid_to_index( &cc->source_pool, sid );
 	if( sindex < 0 ) {
 		/* invalid sid */
@@ -3136,7 +3130,7 @@ ALboolean FL_alUnlockSource( UNUSED(const char *fn),
 	}
 
 	_alLockPrintf("_alUnlockSource", fn, ln);
-	
+
 	sindex = spool_sid_to_index( &cc->source_pool, sid );
 	if( sindex < 0 ) {
 		/* invalid sid */
@@ -3214,6 +3208,7 @@ void *_alGetSourceParam(AL_source *source, ALenum param )
 		case AL_DIRECTION:
 			return &source->direction.data;
 			break;
+		case AL_GAIN:
 		case AL_GAIN_LINEAR_LOKI:
 			return &source->gain.data;
 			break;
@@ -3290,6 +3285,7 @@ ALboolean _alSourceIsParamSet( AL_source *source, ALenum param ) {
 		case AL_DIRECTION:
 			return source->direction.isset;
 			break;
+		case AL_GAIN:
 		case AL_GAIN_LINEAR_LOKI:
 			return source->gain.isset;
 			break;
@@ -3361,6 +3357,7 @@ void _alSourceGetParamDefault( ALenum param, ALvoid *retref ) {
 		case AL_CONE_OUTER_GAIN:
 			*fp = 0.0f;
 			break;
+		case AL_GAIN:
 		case AL_GAIN_LINEAR_LOKI:
 		case AL_MAX_GAIN:
 		case AL_PITCH:
