@@ -162,7 +162,7 @@ unsigned int jlib_debug = 0;
  *
  * Makes the context refered to by handle the current context.  If handle does
  * not refer to a context, ALC_INVALID_CONTEXT is set and returned.
- * Otherwise, the operation sucedes and ALC_NO_ERROR is returned.
+ * Otherwise, the operation suceeds and ALC_NO_ERROR is returned.
  */
 ALCenum alcMakeContextCurrent( ALCcontext *handle )
 {
@@ -193,17 +193,17 @@ ALCenum alcMakeContextCurrent( ALCcontext *handle )
 					_alcSetError( ALC_INVALID_CONTEXT );
 
 					_alcUnlockAllContexts();
-					
+
 					return ALC_INVALID_CONTEXT;
 				}
-				
+
 				/*
 				 * inform current audio device about
 				 * impending stall.
 				 */
 				_alcDevicePause( cc->write_device );
 				_alcDevicePause( cc->read_device );
-				
+
 				_alcCCId = (ALuint) -1;
 				_alcUnlockAllContexts();
 			}
@@ -231,10 +231,10 @@ ALCenum alcMakeContextCurrent( ALCcontext *handle )
 		/* FIXME: wrong error */
 		_alcSetError( ALC_INVALID_CONTEXT );
 		_alcUnlockAllContexts( );
-					
+
 		return ALC_INVALID_CONTEXT;
 	}
-	
+
 	if( should_init_mixer == AL_TRUE ) {
 		/* Set up mixer thread */
 		if(_alInitMixer() == AL_FALSE) {
@@ -247,12 +247,12 @@ ALCenum alcMakeContextCurrent( ALCcontext *handle )
 	{
 		cc->write_device->cc = cc;
 	}
-	
+
 	if(cc->read_device)
 	{
 		cc->read_device->cc = cc;
 	}
-	
+
 	/* set mixer */
 	_alcDeviceWriteSet( cid );
 	_alcDeviceReadSet( cid );
@@ -438,7 +438,7 @@ void alcSuspendContext( ALCcontext *alcHandle )
 
 	if( cc->should_sync == AL_FALSE) {
 		/*
-		 * only asynchronous contexts can be 
+		 * only asynchronous contexts can be
 		 * suspended.
 		 */
 		cc->issuspended = AL_TRUE;
@@ -450,7 +450,7 @@ void alcSuspendContext( ALCcontext *alcHandle )
 }
 
 /*
- * alcCreateContext( struct _AL_device *dev, int *attrlist )
+ * alcCreateContext( struct _AL_device *dev, ALCint *attrlist )
  *
  * Allocates, initialiaes, and returns an AL_context handle, suitable for
  * passing to other alc functions.  Uses dev as the write device for the
@@ -463,13 +463,13 @@ void alcSuspendContext( ALCcontext *alcHandle )
  *
  * FIXME: not as well tested as I'd like.
  */
-ALCcontext *alcCreateContext( struct _AL_device *dev, int *attrlist )
+ALCcontext *alcCreateContext( struct _AL_device *dev, const ALCint *attrlist )
 {
 	ALint cid;
 
 	if( dev == NULL ) {
 		_alcSetError( ALC_INVALID_DEVICE );
-		
+
 		return NULL;
 	}
 
@@ -532,7 +532,7 @@ ALCenum _alcDestroyContext( AL_context *cc )
 	free(cc->Flags);
 	cc->Flags = 0;
 	cc->NumFlags = 0;
-	
+
 	_alDestroyListener( &cc->listener );
 	_alDestroySources( &cc->source_pool );
 
@@ -635,7 +635,7 @@ void _alcSetContext(int *attrlist, ALuint cid, AL_device *dev ) {
 	free(cc->Flags);
 	cc->Flags = 0;
 	cc->NumFlags = 0;
-	
+
 	/* Set our preferred mixer stats */
         if (dev->flags & ALCD_WRITE)
 	{
@@ -679,7 +679,7 @@ void _alcSetContext(int *attrlist, ALuint cid, AL_device *dev ) {
 				break;
 			case ALC_SOURCES_LOKI:
 				spool_resize(&cc->source_pool, rdr.val);
-								
+
 				_alDebug(ALD_CONTEXT,
 					__FILE__, __LINE__,
 					"ALC_SOURCES (%d)", rdr.val);
@@ -839,7 +839,7 @@ ALboolean _alcInUse(ALuint cid) {
 /*
  * FL_alcLockAllContexts( const char *fn, int ln )
  *
- * Locks the mutex associated guarding all contexts, passing fn and ln to 
+ * Locks the mutex associated guarding all contexts, passing fn and ln to
  * _alLockPrintf for debugging purposes.
  */
 void FL_alcLockAllContexts(UNUSED(const char *fn), UNUSED(int ln)) {
@@ -854,7 +854,7 @@ void FL_alcLockAllContexts(UNUSED(const char *fn), UNUSED(int ln)) {
 /*
  * FL_alcUnlockAllContexts( const char *fn, int ln )
  *
- * Unlocks the mutex associated guarding all contexts, passing fn and ln to 
+ * Unlocks the mutex associated guarding all contexts, passing fn and ln to
  * _alLockPrintf for debugging purposes.
  */
 void FL_alcUnlockAllContexts(UNUSED(const char *fn), UNUSED(int ln)) {
@@ -995,7 +995,7 @@ static void _alcReallocContexts(ALuint newsize) {
 		return;
 	}
 
-	/* resize context pool */	
+	/* resize context pool */
 	temp = realloc(al_contexts.pool, sizeof *al_contexts.pool * newsize);
 	if(temp == NULL) {
 		perror("_alcReallocContexts malloc");
@@ -1003,7 +1003,7 @@ static void _alcReallocContexts(ALuint newsize) {
 	}
 	al_contexts.pool = temp;
 
-	/* resize inuse flags */	
+	/* resize inuse flags */
 	temp = realloc(al_contexts.inuse, sizeof *al_contexts.inuse * newsize);
 	if(temp == NULL) {
 		perror("_alcReallocContexts malloc");
@@ -1011,7 +1011,7 @@ static void _alcReallocContexts(ALuint newsize) {
 	}
 	al_contexts.inuse = temp;
 
-	/* resize context map */	
+	/* resize context map */
 	temp = realloc(al_contexts.map, sizeof *al_contexts.map * newsize);
 	if(temp == NULL) {
 		perror("_alcReallocContexts malloc");
@@ -1155,7 +1155,7 @@ static void _alcDeviceReadSet( ALuint cid ) {
 
 	if ( cc->read_device != NULL ) {
 		err = _alcDeviceSet( cc->read_device );
-	
+
 		if( err != AL_TRUE) {
 			_alDebug(ALD_CONTEXT, __FILE__, __LINE__, "set_audiodevice failed.");
 
@@ -1187,11 +1187,12 @@ static void _alcDeviceWriteSet( ALuint cid ) {
 
 	if ( cc->write_device != NULL ) {
 		err = _alcDeviceSet( cc->write_device );
-	
+
 		if( err != AL_TRUE) {
 			_alDebug(ALD_CONTEXT, __FILE__, __LINE__, "set_audiodevice failed.");
 
 			_alcSetError( ALC_INVALID_DEVICE );
+                        return;
 		}
 	}
 
@@ -1351,7 +1352,7 @@ ALsizei _alcDeviceRead( ALuint cid, ALvoid *dataptr, ALuint bytes_to_read )
 		return 0;
 	}
 
-	return capture_audiodevice(cc->read_device->handle, dataptr, bytes_to_read);	
+	return capture_audiodevice(cc->read_device->handle, dataptr, bytes_to_read);
 }
 
 /*
@@ -1417,32 +1418,32 @@ ALboolean _alcIsContextSuspended( ALuint cid ) {
 }
 
 /*
- * alcIsExtensionPresent( UNUSED(ALCdevice *device), ALubyte *extName )
+ * alcIsExtensionPresent( UNUSED(ALCdevice *device), ALCubyte *extName )
  *
  * Returns AL_TRUE if the alc extension extName is present, AL_FALSE
  * otherwise.
  */
-ALboolean alcIsExtensionPresent( UNUSED(ALCdevice *device), ALubyte *extName ) {
+ALCboolean alcIsExtensionPresent( UNUSED(ALCdevice *device), const ALCubyte *extName ) {
 	return alIsExtensionPresent( extName );
 }
 
 /*
- * alcGetProcAddress( UNUSED(ALCdevice *device), ALubyte *funcName ).
+ * alcGetProcAddress( UNUSED(ALCdevice *device), ALCubyte *funcName ).
  *
  * Returns the alc extension function named funcName, or NULL if it doesn't
  * exist.
  */
-ALvoid *alcGetProcAddress( UNUSED(ALCdevice *device), ALubyte *funcName )
+ALCvoid *alcGetProcAddress( UNUSED(ALCdevice *device), const ALCubyte *funcName )
 {
 	return alGetProcAddress( funcName );
 }
 
 /*
- * alcGetEnumValue( ALCdevice *device, ALubyte *enumName )
+ * alcGetEnumValue( ALCdevice *device, ALCubyte *enumName )
  *
  * Returns enum value for enumName.
  */
-ALenum alcGetEnumValue( UNUSED(ALCdevice *device), ALubyte *enumName ) {
+ALCenum alcGetEnumValue( UNUSED(ALCdevice *device), const ALCubyte *enumName ) {
 	return alGetEnumValue( enumName );
 }
 
@@ -1459,19 +1460,19 @@ ALCdevice *alcGetContextsDevice(ALCcontext *handle)
 	{
 		_alcSetError( ALC_INVALID_CONTEXT );
 		_alcUnlockAllContexts( );
-					
+
 		return NULL;
 	}
 
 	dc = cc->write_device;
-	
+
 	/* just unlock contexts */
 	_alcUnlockAllContexts();
 
 	return dc;
 }
 
-const ALubyte *alcGetString( ALCdevice *dev, ALenum token )
+const ALCubyte *alcGetString( ALCdevice *dev, ALCenum token )
 {
 	switch(token)
 	{
@@ -1525,8 +1526,8 @@ AL_context *_alcGetDevicesContext(ALCdevice *deviceHandle)
 	return deviceHandle->cc;
 }
 
-void alcGetIntegerv( ALCdevice *deviceHandle, ALenum  token,
-		     ALsizei  size , ALint *dest )
+void alcGetIntegerv( ALCdevice *deviceHandle, ALCenum  token,
+		     ALCsizei  size , ALCint *dest )
 {
 	AL_context *cc = _alcGetDevicesContext(deviceHandle);
 	if(!cc)
@@ -1538,7 +1539,7 @@ void alcGetIntegerv( ALCdevice *deviceHandle, ALenum  token,
 	{
 		return;
 	}
-		
+
 	switch(token)
 	{
 		  /* JIV FIXME: move major/minor to header
