@@ -18,6 +18,9 @@
  * Or go to http://www.gnu.org/copyleft/lgpl.html
  */
  
+#ifndef _ENVIRONMENT_H_
+#define _ENVIRONMENT_H_
+ 
 #pragma export on
 
 ALAPI ALsizei ALAPIENTRY alGenEnvironmentIASIG (ALsizei n, ALuint *environments);
@@ -26,4 +29,6 @@ ALAPI ALboolean ALAPIENTRY alIsEnvironmentIASIG (ALuint environment);
 ALAPI ALvoid ALAPIENTRY alEnvironmentfIASIG (ALuint environment, ALenum pname, ALfloat value);
 ALAPI ALvoid ALAPIENTRY alEnviromentiIASIG (ALuint environment, ALenum pname, ALint value);
 
-#pragma export off
\ No newline at end of file
+#pragma export off
+
+#endif
