@@ -17,9 +17,9 @@
  *  Boston, MA  02111-1307, USA.
  * Or go to http://www.gnu.org/copyleft/lgpl.html
  */
- 
-// AL_ERROR static info
-static		 ALenum		alLastError =AL_NO_ERROR;
+
+#ifndef _ERROR_H_
+#define _ERROR_H_
 
 #pragma export on
 
@@ -28,3 +28,5 @@ ALAPI ALenum ALAPIENTRY alGetError (ALvoid);
 #pragma export off
 
 ALAPI ALvoid ALAPIENTRY alSetError (ALenum errorCode);
+
+#endif
