@@ -18,10 +18,15 @@
  * Or go to http://www.gnu.org/copyleft/lgpl.html
  */
  
+#ifndef _EXTENSION_H_
+#define _EXTENSION_H_
+
 #pragma export on
 
 ALAPI ALboolean ALAPIENTRY alIsExtensionPresent(ALubyte *extName);
 ALAPI ALvoid * ALAPIENTRY alGetProcAddress(ALubyte *fname);
 ALAPI ALenum ALAPIENTRY alGetEnumValue (ALubyte *ename);
 
-#pragma export off
\ No newline at end of file
+#pragma export off
+
+#endif 
\ No newline at end of file
