@@ -18,20 +18,29 @@
  * Or go to http://www.gnu.org/copyleft/lgpl.html
  */
 
+#if defined(__APPLE__) & defined(__MACH__) // check for OS X
+#define MAC_OS_X
+#else
 #ifdef TARGET_CLASSIC
 #include <Sound.h>
 #else
 #include <Carbon/Carbon.h>
 #endif
+#endif
 
 #include "globaltypes.h"
 
-extern ALbuffer gBuffer[AL_MAXBUFFERS];
-extern ALsource gSource[AL_MAXSOURCES];
+extern ALbuffer gBuffer[AL_MAXBUFFERS]; // holds data on each buffer
+extern ALsource gSource[AL_MAXSOURCES]; // holds data on each source
 extern ALfloat gDopplerFactor;
 extern ALfloat gDopplerVelocity;
 extern ALfloat gDistanceScale;
 extern ALfloat gPropagationSpeed;
 extern ALlistener gListener;
+extern ALuint gBufferSize;
+
+#ifdef MAC_OS_X
+extern void *pVorbisLib;
+#else
 extern SndCallBackUPP gpSMRtn;
-extern ALuint gBufferSize;
\ No newline at end of file
+#endif
