@@ -21,7 +21,6 @@
 #pragma comment(lib, "winmm.lib")
 
 #include <math.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <memory.h>
 #include "OpenAL32/Include/alMain.h"
@@ -29,9 +28,13 @@
 #include "AL/alc.h"
 #include "AL/alu.h"
 #include "OpenAL32/Include/alBuffer.h"
+#ifdef _DEBUG
+	#include "stdio.h"
+#endif
 
 #define SPEEDOFSOUNDMETRESPERSEC	(343.3f)
 #define MAX_NUM_SOURCES			64
+#define OUTPUT_BUFFER_SIZE		32768
 
 typedef struct ALCextension_struct
 {
@@ -46,6 +49,7 @@ typedef struct ALCfunction_struct
 } ALCfunction;
 
 static ALCextension alcExtensions[] = {	
+	{ "ALC_ENUMERATION_EXT",		(ALvoid *) NULL				},
 	{ NULL,							(ALvoid *) NULL				} };
 
 static ALCfunction  alcFunctions[] = {	
@@ -60,7 +64,8 @@ static ALubyte alcErrInvalidValue[] = "Invalid Value";
 
 // Context strings
 static ALubyte alcDefaultDeviceSpecifier[] = "DirectSound3D";
-static ALubyte alcExtensionList[] = "";
+static ALubyte alcDeviceList[] = "DirectSound3D\0DirectSound\0MMSYSTEM\0\0";
+static ALubyte alcExtensionList[] = "Enumeration\0\0";
 
 static ALCint alcMajorVersion = 1;
 static ALCint alcMinorVersion = 0;
@@ -87,6 +92,7 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource);
 void UpdateListener(ALCcontext *ALContext);
 
 // Local function definitions
+void EAXFix(ALCcontext *context);
 ALuint GetMaxNumStereoBuffers(LPDIRECTSOUND lpDS);
 ALuint GetMaxNum3DMonoBuffers(LPDIRECTSOUND lpDS);
 
@@ -147,71 +153,57 @@ ALCAPI ALCvoid ALCAPIENTRY alcProcessContext(ALCcontext *context)
 	// Not a lot happens here !
 }
 
-ALCAPI ALCubyte* ALCAPIENTRY alcGetString(ALCdevice *device,ALCenum param)
+ALCAPI const ALCubyte* ALCAPIENTRY alcGetString(ALCdevice *device,ALCenum param)
 {
-	ALCcontext *Context;
-	ALubyte *value = NULL;
+	const ALubyte *value = NULL;
 
-	Context=alcGetCurrentContext();
-	if (Context)
+	switch(param)
 	{
-		SuspendContext(Context);
-	
-		switch(param)
-		{
-		case ALC_NO_ERROR:
-			value=alcNoError;
-			break;
+	case ALC_NO_ERROR:
+		value=alcNoError;
+		break;
 
-		case ALC_INVALID_ENUM:
-			value=alcErrInvalidEnum;
-			break;
+	case ALC_INVALID_ENUM:
+		value=alcErrInvalidEnum;
+		break;
 
-		case ALC_INVALID_VALUE:
-			value=alcErrInvalidValue;
-			break;
+	case ALC_INVALID_VALUE:
+		value=alcErrInvalidValue;
+		break;
 
-		case ALC_INVALID_DEVICE:
-			value=alcErrInvalidDevice;
-			break;
+	case ALC_INVALID_DEVICE:
+		value=alcErrInvalidDevice;
+		break;
 
-		case ALC_INVALID_CONTEXT:
-			value=alcErrInvalidContext;
-			break;
+	case ALC_INVALID_CONTEXT:
+		value=alcErrInvalidContext;
+		break;
 
-		case ALC_DEFAULT_DEVICE_SPECIFIER:
-			value = alcDefaultDeviceSpecifier;
-			break;
+	case ALC_DEFAULT_DEVICE_SPECIFIER:
+		value = alcDefaultDeviceSpecifier;
+		break;
 
-		case ALC_DEVICE_SPECIFIER:
-			if (!device)
-				alcSetError(ALC_INVALID_DEVICE);
-			else
-				value = device->szDeviceName;
-			break;
+	case ALC_DEVICE_SPECIFIER:
+		if (device)
+			value = device->szDeviceName;
+		else
+			value = alcDeviceList;
+		break;
 
-		case ALC_EXTENSIONS:
-			value = alcExtensionList;
-			break;
+	case ALC_EXTENSIONS:
+		value = alcExtensionList;
+		break;
 
-		default:
-			alcSetError(ALC_INVALID_ENUM);
-			break;
-		}
-	
-		ProcessContext(Context);
-	}
-	else
-	{
-		// Invalid Context
-		alcSetError(ALC_INVALID_CONTEXT);
+	default:
+		alcSetError(ALC_INVALID_ENUM);
+		break;
 	}
 
 	return value;
 }
 
 
-ALCAPI ALCvoid ALCAPIENTRY alcGetIntegerv(ALCdevice *device,ALCenum param,ALCsizei size,ALCint *data)
+ALCAPI ALCvoid ALCAPIENTRY alcGetIntegerv(ALCdevice *device,ALCenum param,ALsizei size,ALCint *data)
 {
 	ALCcontext *Context;
 
@@ -354,6 +346,7 @@ static void CALLBACK alcDirectSoundProc(UINT uID,UINT uReserved,DWORD dwUser,DWO
 	DWORD PlayCursor,WriteCursor;
 	BYTE *WritePtr1,*WritePtr2;
 	DWORD WriteCnt1,WriteCnt2;
+	WAVEFORMATEX OutputType;
 	ALCcontext *ALContext;
 	ALCdevice *ALDevice;
 	DWORD BytesPlayed;
@@ -363,18 +356,30 @@ static void CALLBACK alcDirectSoundProc(UINT uID,UINT uReserved,DWORD dwUser,DWO
 	SuspendContext(ALContext);
 
 	ALDevice=(ALCdevice *)dwUser;
+
+	// Get current play and write cursors
 	IDirectSoundBuffer_GetCurrentPosition(ALDevice->DSsbuffer,&PlayCursor,&WriteCursor);
 	if (!OldWriteCursor) OldWriteCursor=WriteCursor-PlayCursor;
-	BytesPlayed=((((long)WriteCursor-(long)OldWriteCursor)<0)?((long)32768+(long)WriteCursor-(long)OldWriteCursor):((DWORD)WriteCursor-(DWORD)OldWriteCursor));
-	DSRes=IDirectSoundBuffer_Lock(ALDevice->DSsbuffer,(OldWriteCursor+3528)&32767,BytesPlayed,&WritePtr1,&WriteCnt1,&WritePtr2,&WriteCnt2,0);
+
+	// Get the output format and figure the number of bytes played (block aligned)
+	IDirectSoundBuffer_GetFormat(ALDevice->DSsbuffer,&OutputType,sizeof(WAVEFORMATEX),NULL);
+	BytesPlayed=((((WriteCursor<OldWriteCursor)?(OUTPUT_BUFFER_SIZE+WriteCursor-OldWriteCursor):(WriteCursor-OldWriteCursor))/OutputType.nBlockAlign)*OutputType.nBlockAlign);
+
+	// Lock output buffer started at 40msec in front of the old write cursor (15msec in front of the actual write cursor)
+	DSRes=IDirectSoundBuffer_Lock(ALDevice->DSsbuffer,(OldWriteCursor+(OutputType.nSamplesPerSec/25)*OutputType.nBlockAlign)%OUTPUT_BUFFER_SIZE,BytesPlayed,&WritePtr1,&WriteCnt1,&WritePtr2,&WriteCnt2,0);
+
+	// If the buffer is lost, restore it, play and lock
 	if (DSRes==DSERR_BUFFERLOST)
 	{
 		IDirectSoundBuffer_Restore(ALDevice->DSsbuffer);
 		IDirectSoundBuffer_Play(ALDevice->DSsbuffer,0,0,DSBPLAY_LOOPING);
-		DSRes=IDirectSoundBuffer_Lock(ALDevice->DSsbuffer,(OldWriteCursor+3528)&32767,BytesPlayed,&WritePtr1,&WriteCnt1,&WritePtr2,&WriteCnt2,0);
+		DSRes=IDirectSoundBuffer_Lock(ALDevice->DSsbuffer,(OldWriteCursor+(OutputType.nSamplesPerSec/25)*OutputType.nBlockAlign)%OUTPUT_BUFFER_SIZE,BytesPlayed,&WritePtr1,&WriteCnt1,&WritePtr2,&WriteCnt2,0);
 	}
+
+	// Successfully locked the output buffer
 	if (DSRes==DS_OK)
 	{
+		// If we have a active context, mix data directly into output buffer otherwise fill with silence
 		if (ALContext)
 		{
 			if (WritePtr1)
@@ -389,9 +394,13 @@ static void CALLBACK alcDirectSoundProc(UINT uID,UINT uReserved,DWORD dwUser,DWO
 			if (WritePtr2)
 				memset(WritePtr2,0,WriteCnt2);
 		}
+
+		// Unlock output buffer only when successfully locked
 		IDirectSoundBuffer_Unlock(ALDevice->DSsbuffer,WritePtr1,WriteCnt1,WritePtr2,WriteCnt2);
 	}
-	OldWriteCursor=WriteCursor;
+	
+	// Update old write cursor location
+	OldWriteCursor=((OldWriteCursor+BytesPlayed)%OUTPUT_BUFFER_SIZE);
 
 	ProcessContext(ALContext);
 }
@@ -419,6 +428,21 @@ ALvoid alcInitContext(ALCcontext *context)
 		context->Listener.Up[1]=1.0f;
 		context->Listener.Up[2]=0.0f;
 
+		context->Listener.EAX20LP.dwEnvironment = EAX_ENVIRONMENT_GENERIC;
+		context->Listener.EAX20LP.flEnvironmentSize = EAXLISTENER_DEFAULTENVIRONMENTSIZE;
+		context->Listener.EAX20LP.flEnvironmentDiffusion = EAXLISTENER_DEFAULTENVIRONMENTDIFFUSION;
+		context->Listener.EAX20LP.lRoom = EAXLISTENER_DEFAULTROOM;
+		context->Listener.EAX20LP.lRoomHF = EAXLISTENER_DEFAULTROOMHF;
+		context->Listener.EAX20LP.flDecayTime = EAXLISTENER_DEFAULTDECAYTIME;
+		context->Listener.EAX20LP.flDecayHFRatio = EAXLISTENER_DEFAULTDECAYHFRATIO;
+		context->Listener.EAX20LP.lReflections = EAXLISTENER_DEFAULTREFLECTIONS;
+		context->Listener.EAX20LP.flReflectionsDelay = EAXLISTENER_DEFAULTREFLECTIONSDELAY;
+		context->Listener.EAX20LP.lReverb = EAXLISTENER_DEFAULTREVERB;
+		context->Listener.EAX20LP.flReverbDelay = EAXLISTENER_DEFAULTREVERBDELAY;
+		context->Listener.EAX20LP.flAirAbsorptionHF = EAXLISTENER_DEFAULTAIRABSORPTIONHF;
+		context->Listener.EAX20LP.flRoomRolloffFactor = EAXLISTENER_DEFAULTROOMROLLOFFFACTOR;
+		context->Listener.EAX20LP.dwFlags = EAXLISTENER_DEFAULTFLAGS;
+
 		//Validate context
 		context->LastError=AL_NO_ERROR;
 		context->InUse=AL_FALSE;
@@ -475,7 +499,7 @@ ALCvoid alcExitContext(ALCcontext *context)
 		// End Timer Period
 		timeEndPeriod(g_nTimerInterval);
 
-#ifdef _DEBUG		
+#ifdef _DEBUG
 		if (context->SourceCount>0)
 		{
 			sprintf(szString,"OpenAL32 : alcDestroyContext() %d Source(s) NOT deleted\n", context->SourceCount);
@@ -510,6 +534,32 @@ ALCvoid alcExitContext(ALCcontext *context)
 			ALSource = ALTempSource;
 		}
 
+		// If we created the Permanent Source ( in EAXFix() ), then manually delete it now
+		if (context->alPrivateSource)
+		{
+			ALSource=((ALsource *)context->alPrivateSource);
+
+			// Delete the Source
+			if (ALSource->uservalue1)
+			{
+				IDirectSoundBuffer_Stop((LPDIRECTSOUNDBUFFER)ALSource->uservalue1);
+				if (ALSource->uservalue3)
+				{
+					IKsPropertySet_Release((LPKSPROPERTYSET)ALSource->uservalue3);
+					ALSource->uservalue3 = NULL;
+				}
+				if (ALSource->uservalue2)
+				{
+					IDirectSound3DBuffer_Release((LPDIRECTSOUND3DBUFFER)ALSource->uservalue2);
+					ALSource->uservalue2 = NULL;
+				}
+				IDirectSoundBuffer_Release((LPDIRECTSOUNDBUFFER)ALSource->uservalue1);
+				ALSource->uservalue1=NULL;
+			}
+			memset(ALSource,0,sizeof(ALsource));
+			free(ALSource);
+		}
+
 		//Invalidate context
 		context->LastError=AL_NO_ERROR;
 		context->InUse=AL_FALSE;
@@ -523,7 +573,7 @@ ALCvoid alcExitContext(ALCcontext *context)
 }
 
 
-ALCAPI ALCcontext*ALCAPIENTRY alcCreateContext(ALCdevice *device,ALCint *attrList)
+ALCAPI ALCcontext*ALCAPIENTRY alcCreateContext(ALCdevice *device, const ALCint *attrList)
 {
 	ALCcontext *ALContext = NULL;
 
@@ -651,6 +701,12 @@ ALCAPI ALCboolean ALCAPIENTRY alcMakeContextCurrent(ALCcontext *context)
 		{
 			SuspendContext(ALContext);
 			ALContext->InUse=AL_TRUE;
+
+			if (ALContext->Device->DS3dlistener)
+			{
+				EAXFix(context);
+			}
+
 			ProcessContext(ALContext);
 		}
 	}
@@ -663,7 +719,6 @@ ALCAPI ALCboolean ALCAPIENTRY alcMakeContextCurrent(ALCcontext *context)
 	return bReturn;
 }
 
-
 void CALLBACK TimerCallback(UINT uID, UINT uMsg, DWORD dwUser, DWORD dw1, DWORD dw2)
 {
 	ALuint i, loop;
@@ -700,7 +755,6 @@ void CALLBACK TimerCallback(UINT uID, UINT uMsg, DWORD dwUser, DWORD dw1, DWORD
 			//	 bytesplayed += 88200
 			//   oldwritecursor = writecursor
 			// oldtime = current time
-
 			NewTime = timeGetTime();
 
 			if ((NewTime - ALSource->OldTime) > ALSource->BufferDuration)
@@ -730,26 +784,40 @@ void CALLBACK TimerCallback(UINT uID, UINT uMsg, DWORD dwUser, DWORD dw1, DWORD
 			// If Source is PAUSED or STOPPED, copy silence into DS Buffer
 			if ((ALSource->CurrentState == AL_PAUSED) || (ALSource->CurrentState == AL_STOPPED))
 			{
-				IDirectSoundBuffer_Lock((LPDIRECTSOUNDBUFFER)ALSource->uservalue1, ALSource->OldWriteCursor, DataToLock, &lpPart1, &Part1Size, &lpPart2, &Part2Size, 0);
+				// Check if we have already filled the buffer with silence
 
-				if (lpPart1)
+				if (ALSource->Silence < 88200)
 				{
-					memset(lpPart1, 0, Part1Size);
-					ALSource->OldWriteCursor += Part1Size;
+					IDirectSoundBuffer_Lock((LPDIRECTSOUNDBUFFER)ALSource->uservalue1, ALSource->OldWriteCursor, DataToLock, &lpPart1, &Part1Size, &lpPart2, &Part2Size, 0);
+
+					if (lpPart1)
+					{
+						memset(lpPart1, 0, Part1Size);
+						ALSource->Silence += Part1Size;
+						ALSource->OldWriteCursor += Part1Size;
+						if (ALSource->OldWriteCursor >= 88200)
+							ALSource->OldWriteCursor -= 88200;
+					}
+
+					if (lpPart2)
+					{
+						memset(lpPart2, 0, Part2Size);
+						ALSource->Silence += Part2Size;
+						ALSource->OldWriteCursor += Part2Size;
+						if (ALSource->OldWriteCursor >= 88200)
+							ALSource->OldWriteCursor -= 88200;
+					}
+
+					if (FAILED(IDirectSoundBuffer_Unlock((LPDIRECTSOUNDBUFFER)ALSource->uservalue1, lpPart1, Part1Size, lpPart2, Part2Size)))
+						OutputDebugString("FAILED UNLOCK!!!\n");
+				}
+				else
+				{
+					ALSource->OldWriteCursor += DataToLock;
 					if (ALSource->OldWriteCursor >= 88200)
 						ALSource->OldWriteCursor -= 88200;
 				}
 
-				if (lpPart2)
-				{
-					memset(lpPart2, 0, Part2Size);
-					ALSource->OldWriteCursor += Part2Size;
-					if (ALSource->OldWriteCursor >= 88200)
-						ALSource->OldWriteCursor -= 88200;
-				}
-
-				IDirectSoundBuffer_Unlock((LPDIRECTSOUNDBUFFER)ALSource->uservalue1, lpPart1, Part1Size, lpPart2, Part2Size);
-
 				// Update Old Play Cursor
 				ALSource->OldPlayCursor = PlayCursor;
 
@@ -1087,7 +1155,8 @@ void CALLBACK TimerCallback(UINT uID, UINT uMsg, DWORD dwUser, DWORD dw1, DWORD
 
 				ALSource->OldPlayCursor = PlayCursor;
 
-				IDirectSoundBuffer_Unlock((LPDIRECTSOUNDBUFFER)ALSource->uservalue1, lpPart1, Part1Size, lpPart2, Part2Size);				
+				IDirectSoundBuffer_Unlock((LPDIRECTSOUNDBUFFER)ALSource->uservalue1, lpPart1, Part1Size, lpPart2, Part2Size);
+
 			} // end else
 
 			// If we are still playing (we may have reached the end) then update current buffer
@@ -1209,7 +1278,7 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 		memset(&DSBDescription,0,sizeof(DSBUFFERDESC));
 		DSBDescription.dwSize=sizeof(DSBUFFERDESC);
 		DSBDescription.dwFlags=DSBCAPS_CTRLVOLUME|DSBCAPS_CTRLFREQUENCY|DSBCAPS_CTRL3D|DSBCAPS_GLOBALFOCUS|
-			DSBCAPS_GETCURRENTPOSITION2|ALContext->Device->CreationFlag;
+			DSBCAPS_GETCURRENTPOSITION2|DSBCAPS_LOCHARDWARE;
 		DSBDescription.dwBufferBytes=88200;
 		DSBDescription.lpwfxFormat=&OutputType;
 		memset(&OutputType,0,sizeof(WAVEFORMATEX));
@@ -1220,6 +1289,7 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 		OutputType.nSamplesPerSec=44100;
 		OutputType.nAvgBytesPerSec=88200;
 		OutputType.cbSize=0;
+		ALSource->DSFrequency=44100;
 		if (IDirectSound_CreateSoundBuffer(ALContext->Device->DShandle,&DSBDescription,(LPDIRECTSOUNDBUFFER *)&ALSource->uservalue1,NULL)==DS_OK)
 		{
 			IDirectSoundBuffer_SetCurrentPosition((LPDIRECTSOUNDBUFFER)ALSource->uservalue1,0);
@@ -1268,7 +1338,7 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 				}
 
 				if (ALSource->play)
-				{	
+				{
 					// If Source is already playing, we need to restart it from the beginning
 					if (ALSource->CurrentState == AL_PLAYING)
 					{	
@@ -1279,15 +1349,17 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 							ALBufferListItem->bufferstate = PENDING;
 							ALBufferListItem = ALBufferListItem->next;
 						}
-
-						ALSource->BuffersProcessed = 0;
-						ALSource->BuffersAddedToDSBuffer = 0;
-						ALSource->BufferPosition = 0;
-						ALSource->BytesPlayed = 0;
-						ALSource->DataStillToPlay = 0;
-						ALSource->FinishedQueue = AL_FALSE;
 					}
 
+					// Always reset these variables no matter whether Source WAS playing or not
+					ALSource->BuffersProcessed = 0;
+					ALSource->BuffersAddedToDSBuffer = 0;
+					ALSource->BufferPosition = 0;
+					ALSource->BytesPlayed = 0;
+					ALSource->DataStillToPlay = 0;
+					ALSource->FinishedQueue = AL_FALSE;
+					ALSource->Silence = 0;
+
 					// If the queue is empty, set Source State to STOPPED
 					if (ALSource->BuffersInQueue == 0)
 					{
@@ -1420,15 +1492,17 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 						}
 						IDirectSoundBuffer_Stop((LPDIRECTSOUNDBUFFER)ALSource->uservalue1);
 						IDirectSoundBuffer_Release((LPDIRECTSOUNDBUFFER)ALSource->uservalue1);
+						
 						ALSource->uservalue1=NULL;
-
+						ALSource->DSBufferPlaying = AL_FALSE;
+						
 						ALSource->SourceType = SOURCE2D;
 
 						// Set Caps
 						memset(&DSBDescription,0,sizeof(DSBUFFERDESC));
 						DSBDescription.dwSize=sizeof(DSBUFFERDESC);
 						DSBDescription.dwFlags=DSBCAPS_CTRLVOLUME|DSBCAPS_CTRLFREQUENCY|DSBCAPS_GLOBALFOCUS|
-							DSBCAPS_GETCURRENTPOSITION2|ALContext->Device->CreationFlag;
+							DSBCAPS_GETCURRENTPOSITION2|DSBCAPS_LOCSOFTWARE;
 						DSBDescription.dwBufferBytes=88200;
 						DSBDescription.lpwfxFormat=&OutputType;
 						memset(&OutputType,0,sizeof(WAVEFORMATEX));
@@ -1454,8 +1528,6 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 						}
 
 						// Update variables
-						ALSource->DSBufferPlaying = AL_FALSE;
-
 						ALSource->OldPlayCursor = 0;
 						ALSource->OldWriteCursor = 0;
 
@@ -1478,13 +1550,16 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 							IDirectSoundBuffer_Stop((LPDIRECTSOUNDBUFFER)ALSource->uservalue1);
 							IDirectSoundBuffer_Release((LPDIRECTSOUNDBUFFER)ALSource->uservalue1);
 							ALSource->uservalue1=NULL;
+							ALSource->DSBufferPlaying = AL_FALSE;
 						}
 
+						ALSource->SourceType = SOURCE3D;
+
 						// Set Caps
 						memset(&DSBDescription,0,sizeof(DSBUFFERDESC));
 						DSBDescription.dwSize=sizeof(DSBUFFERDESC);
 						DSBDescription.dwFlags=DSBCAPS_CTRLVOLUME|DSBCAPS_CTRLFREQUENCY|DSBCAPS_CTRL3D|DSBCAPS_GLOBALFOCUS|
-							DSBCAPS_GETCURRENTPOSITION2|ALContext->Device->CreationFlag;
+							DSBCAPS_GETCURRENTPOSITION2|DSBCAPS_LOCHARDWARE;
 						DSBDescription.dwBufferBytes=88200;
 						DSBDescription.lpwfxFormat=&OutputType;
 						memset(&OutputType,0,sizeof(WAVEFORMATEX));
@@ -1518,8 +1593,6 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 						}
 
 						// Update variables
-						ALSource->DSBufferPlaying = AL_FALSE;
-
 						ALSource->OldPlayCursor = 0;
 						ALSource->OldWriteCursor = 0;
 
@@ -1531,15 +1604,17 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 							volume = LinearGainToDB(Gain);
 							IDirectSoundBuffer_SetVolume((LPDIRECTSOUNDBUFFER)ALSource->uservalue1, volume);
 						}
-
-						ALSource->SourceType = SOURCE3D;
 					}
 
 					// Set Direct Sound buffer to frequency of current Open AL buffer multiplied by desired Pitch
 					Freq = ((ALbuffer*)BufferID)->frequency;
 					Pitch = ALSource->param[AL_PITCH-AL_CONE_INNER_ANGLE].data.f;
 
-					IDirectSoundBuffer_SetFrequency((LPDIRECTSOUNDBUFFER)ALSource->uservalue1, (long)(Freq*Pitch));
+					if (ALSource->DSFrequency != (unsigned long)(Freq*Pitch))
+					{
+						IDirectSoundBuffer_SetFrequency((LPDIRECTSOUNDBUFFER)ALSource->uservalue1, (unsigned long)(Freq*Pitch));
+						ALSource->DSFrequency = (unsigned long)(Freq*Pitch);
+					}
 
 					// Record duration of the DS circular buffer
 					if (ALSource->SourceType == SOURCE3D)
@@ -1558,7 +1633,8 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 						else
 							DataToCopy = ((88200 - WriteCursor) + PlayCursor);
 
-						IDirectSoundBuffer_Lock((LPDIRECTSOUNDBUFFER)ALSource->uservalue1,WriteCursor,DataToCopy,&lpPart1,&Part1Size,&lpPart2,&Part2Size,0);
+						if (FAILED(IDirectSoundBuffer_Lock((LPDIRECTSOUNDBUFFER)ALSource->uservalue1,WriteCursor,DataToCopy,&lpPart1,&Part1Size,&lpPart2,&Part2Size,0)))
+							OutputDebugString("LOCK FAILED!!!!\n");
 						
 						if (lpPart1 != NULL)
 						{
@@ -1648,19 +1724,17 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 								memset((ALubyte*)lpPart1 + BytesWritten, 0, Part1Size - BytesWritten);
 								ALSource->SilenceAdded += (Part1Size - BytesWritten);
 							}
-
-							ALSource->OldWriteCursor += Part1Size;
-							if (ALSource->OldWriteCursor >= 88200)
-								ALSource->OldWriteCursor -= 88200;
 						}
 
-
 						if (lpPart2 != NULL)
 						{
 							if (ALSource->FinishedQueue)
 							{
 								// Fill Part 2 with silence
 								memset(lpPart2, 0, Part2Size);
+
+
+								ALSource->SilenceAdded += Part2Size;
 							}
 							else
 							{
@@ -1749,21 +1823,22 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 									memset((ALubyte*)lpPart2 + BytesWritten, 0, Part2Size - BytesWritten);
 									ALSource->SilenceAdded += (Part2Size - BytesWritten);
 								}
-
-								ALSource->OldWriteCursor += Part2Size;
-								if (ALSource->OldWriteCursor >= 88200)
-									ALSource->OldWriteCursor -= 88200;
 							}
 						}
 
+						// We will have filled the whole buffer (minus gap between play and write cursors) with data
+						// up to the new Play Cursor.  So set OldWrite and OldPlay to current Play cursor, for the next
+						// timer callback to correctly service this source.
 						ALSource->OldPlayCursor = PlayCursor;
+						ALSource->OldWriteCursor = PlayCursor;
 
-						IDirectSoundBuffer_Unlock((LPDIRECTSOUNDBUFFER)ALSource->uservalue1, lpPart1, Part1Size, lpPart2, Part2Size);
-
+						if(FAILED(IDirectSoundBuffer_Unlock((LPDIRECTSOUNDBUFFER)ALSource->uservalue1, lpPart1, Part1Size, lpPart2, Part2Size)))
+							OutputDebugString("FAILED UNLOCK!!!\n");
 					}
 					else
 					{
-						IDirectSoundBuffer_Lock((LPDIRECTSOUNDBUFFER)ALSource->uservalue1,0,0,&lpPart1,&Part1Size,0,0,DSBLOCK_ENTIREBUFFER);
+						if (FAILED(IDirectSoundBuffer_Lock((LPDIRECTSOUNDBUFFER)ALSource->uservalue1,0,0,&lpPart1,&Part1Size,0,0,DSBLOCK_ENTIREBUFFER)))
+							OutputDebugString("FAILED LOCK!!!!\n");
 											
 						BytesWritten = 0;
 
@@ -1828,7 +1903,8 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 							ALSource->FinishedQueue = AL_TRUE;		// Set this to true to indicate no more data needs to be copied into buffer
 						}
 
-						IDirectSoundBuffer_Unlock((LPDIRECTSOUNDBUFFER)ALSource->uservalue1,lpPart1,Part1Size,0,0);
+						if (FAILED(IDirectSoundBuffer_Unlock((LPDIRECTSOUNDBUFFER)ALSource->uservalue1,lpPart1,Part1Size,0,0)))
+							OutputDebugString("FAILED UNLOCK!!!\n");
 					}
 				}
 
@@ -1851,7 +1927,8 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 						else
 							DataToCopy = ((88200 - WriteCursor) + PlayCursor);
 
-						IDirectSoundBuffer_Lock((LPDIRECTSOUNDBUFFER)ALSource->uservalue1,WriteCursor,DataToCopy,&lpPart1,&Part1Size,&lpPart2,&Part2Size,0);
+						if (FAILED(IDirectSoundBuffer_Lock((LPDIRECTSOUNDBUFFER)ALSource->uservalue1,WriteCursor,DataToCopy,&lpPart1,&Part1Size,&lpPart2,&Part2Size,0)))
+							OutputDebugString("FAILED LOCK!\n");
 						
 						if (lpPart1 != NULL)
 						{
@@ -1871,7 +1948,8 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 
 						ALSource->OldPlayCursor = PlayCursor;
 
-						IDirectSoundBuffer_Unlock((LPDIRECTSOUNDBUFFER)ALSource->uservalue1, lpPart1, Part1Size, lpPart2, Part2Size);
+						if (FAILED(IDirectSoundBuffer_Unlock((LPDIRECTSOUNDBUFFER)ALSource->uservalue1, lpPart1, Part1Size, lpPart2, Part2Size)))
+							OutputDebugString("FAILED UNLOCK!!!\n");
 					}
 				}
 				break;
@@ -1897,7 +1975,8 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 						else
 							DataToCopy = ((88200 - WriteCursor) + PlayCursor);
 
-						IDirectSoundBuffer_Lock((LPDIRECTSOUNDBUFFER)ALSource->uservalue1,WriteCursor,DataToCopy,&lpPart1,&Part1Size,&lpPart2,&Part2Size,0);
+						if (FAILED(IDirectSoundBuffer_Lock((LPDIRECTSOUNDBUFFER)ALSource->uservalue1,WriteCursor,DataToCopy,&lpPart1,&Part1Size,&lpPart2,&Part2Size,0)))
+							OutputDebugString("FAILED LOCK!!!!\n");
 						
 						if (lpPart1 != NULL)
 						{
@@ -1917,7 +1996,8 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 
 						ALSource->OldPlayCursor = PlayCursor;
 
-						IDirectSoundBuffer_Unlock((LPDIRECTSOUNDBUFFER)ALSource->uservalue1, lpPart1, Part1Size, lpPart2, Part2Size);
+						if (FAILED(IDirectSoundBuffer_Unlock((LPDIRECTSOUNDBUFFER)ALSource->uservalue1, lpPart1, Part1Size, lpPart2, Part2Size)))
+							OutputDebugString("FAILED UNLOCK!!!\n");
 
 						// Mark all buffers in queue as PROCESSED
 						ALSource->BuffersProcessed = ALSource->BuffersInQueue;
@@ -1949,6 +2029,7 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 			Pos[1] = ALSource->param[AL_POSITION-AL_CONE_INNER_ANGLE].data.fv3[1];
 			Pos[2] = -ALSource->param[AL_POSITION-AL_CONE_INNER_ANGLE].data.fv3[2];
 			IDirectSound3DBuffer_SetPosition((LPDIRECTSOUND3DBUFFER)ALSource->uservalue2,Pos[0],Pos[1],Pos[2],DS3D_IMMEDIATE);
+
 			ALSource->update1 &= ~POSITION;
 			if (ALSource->update1 == 0)
 				return;
@@ -1965,6 +2046,7 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 			Vel[1] = ALSource->param[AL_VELOCITY-AL_CONE_INNER_ANGLE].data.fv3[1];
 			Vel[2] = -ALSource->param[AL_VELOCITY-AL_CONE_INNER_ANGLE].data.fv3[2];
 			IDirectSound3DBuffer_SetVelocity((LPDIRECTSOUND3DBUFFER)ALSource->uservalue2,Vel[0],Vel[1],Vel[2],DS3D_IMMEDIATE);
+
 			ALSource->update1 &= ~VELOCITY;
 			if (ALSource->update1 == 0)
 				return;
@@ -1981,6 +2063,7 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 			Dir[1] = ALSource->param[AL_DIRECTION-AL_CONE_INNER_ANGLE].data.fv3[1];
 			Dir[2] = -ALSource->param[AL_DIRECTION-AL_CONE_INNER_ANGLE].data.fv3[2];
 			IDirectSound3DBuffer_SetConeOrientation((LPDIRECTSOUND3DBUFFER)ALSource->uservalue2,Dir[0],Dir[1],Dir[2],DS3D_IMMEDIATE);
+
 			ALSource->update1 &= ~ORIENTATION;
 			if (ALSource->update1 == 0)
 				return;
@@ -1992,19 +2075,19 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 	if (ALSource->update1 & SQUEUE)
 	{
 		if ((ALSource->uservalue1) && (ALSource->state == AL_PLAYING))
-		{			
+		{	
 			// Some buffer(s) have been added to the queue
 
 			// If silence has been added, then we need to overwrite the silence with new audio
 			// data from the buffers in the queue
 			if (ALSource->SilenceAdded > 0)
 			{
-				// Find position of end of valid data
-				if (ALSource->OldWriteCursor < ALSource->SilenceAdded)
-					ALSource->OldWriteCursor += 88200;
-
-				// Set new Write position to the end of the valid data
-				ALSource->OldWriteCursor -= ALSource->SilenceAdded;
+				if (ALSource->SilenceAdded > ALSource->OldWriteCursor)
+				{
+					ALSource->OldWriteCursor = (88200 - ALSource->SilenceAdded + ALSource->OldWriteCursor);
+				}
+				else
+					ALSource->OldWriteCursor -= ALSource->SilenceAdded;
 
 				// Read position from next buffer should be set to 0
 				ALSource->BufferPosition = 0;
@@ -2015,6 +2098,47 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 				// Make sure that the we haven't finished processing the queue !
 				ALSource->FinishedQueue = AL_FALSE;
 			}
+			else if (ALSource->param[AL_LOOPING-AL_CONE_INNER_ANGLE].data.i == AL_TRUE)
+			{
+				// Get position in DS Buffer
+				IDirectSoundBuffer_GetCurrentPosition((LPDIRECTSOUNDBUFFER)ALSource->uservalue1, &PlayCursor, &WriteCursor);
+
+				// Calculate amount of data played
+				if (ALSource->OldPlayCursor > PlayCursor)
+					ALSource->BytesPlayed += ((88200 - ALSource->OldPlayCursor) + PlayCursor);
+				else
+					ALSource->BytesPlayed += (PlayCursor - ALSource->OldPlayCursor);
+
+				// Calculate offset position in queue
+				ALSource->BytesPlayed = (ALSource->BytesPlayed % ALSource->TotalBufferDataSize);
+
+				// Calculate data to play
+				ALSource->DataStillToPlay = ALSource->TotalBufferDataSize - ALSource->BytesPlayed;
+
+				// Make sure we have at least 50ms (timer interupt period) of data
+				if (ALSource->DataStillToPlay < (int)((88200.0f*50.0f)/(float)ALSource->BufferDuration))
+				{					
+					ALSource->DataStillToPlay += ALSource->TotalBufferDataSize;
+				}
+
+				if (ALSource->DataStillToPlay < 88200)
+				{
+					// Need to find position to write new data
+					ALSource->OldWriteCursor = PlayCursor + ALSource->DataStillToPlay;
+					if (ALSource->OldWriteCursor >= 88200)
+						ALSource->OldWriteCursor -= 88200;
+
+					// Read position from next buffer should be set to 0
+					ALSource->BufferPosition = 0;
+
+					ALSource->BuffersAddedToDSBuffer = ALSource->BuffersInQueue - ALSource->NumBuffersAddedToQueue;
+
+					// Make sure that the we haven't finished processing the queue !
+					ALSource->FinishedQueue = AL_FALSE;
+					ALSource->OldPlayCursor = PlayCursor;
+				}
+			}
+
 			// Update DataStillToPlay
 			ALSource->DataStillToPlay += ALSource->SizeOfBufferDataAddedToQueue;
 		}
@@ -2072,7 +2196,9 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 			Gain = ALSource->param[AL_GAIN-AL_CONE_INNER_ANGLE].data.f;
 			Gain = (Gain * ALContext->Listener.Gain);
 			volume = LinearGainToDB(Gain);
+
 			IDirectSoundBuffer_SetVolume((LPDIRECTSOUNDBUFFER)ALSource->uservalue1, volume);
+			
 			ALSource->update1 &= ~VOLUME;
 			if (ALSource->update1 == 0)
 				return;
@@ -2093,7 +2219,8 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 
 			Pitch = ALSource->param[AL_PITCH-AL_CONE_INNER_ANGLE].data.f;
 			
-			IDirectSoundBuffer_SetFrequency((LPDIRECTSOUNDBUFFER)ALSource->uservalue1,(long)(Freq*Pitch));
+			ALSource->DSFrequency = (unsigned long)(Freq*Pitch);
+			IDirectSoundBuffer_SetFrequency((LPDIRECTSOUNDBUFFER)ALSource->uservalue1,ALSource->DSFrequency);
 			
 			// Update duration of the DS circular buffer
 			if (ALSource->SourceType == SOURCE3D)
@@ -2115,6 +2242,7 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 		{
 			minDist = ALSource->param[AL_REFERENCE_DISTANCE-AL_CONE_INNER_ANGLE].data.f;
 			IDirectSound3DBuffer_SetMinDistance((LPDIRECTSOUND3DBUFFER)ALSource->uservalue2,minDist,DS3D_IMMEDIATE);
+
 			ALSource->update1 &= ~MINDIST;
 			if (ALSource->update1 == 0)
 				return;
@@ -2129,6 +2257,7 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 		{
 			maxDist = ALSource->param[AL_MAX_DISTANCE-AL_CONE_INNER_ANGLE].data.f;
 			IDirectSound3DBuffer_SetMaxDistance((LPDIRECTSOUND3DBUFFER)ALSource->uservalue2,maxDist,DS3D_IMMEDIATE);
+			
 			ALSource->update1 &= ~MAXDIST;
 			if (ALSource->update1 == 0)
 				return;
@@ -2145,6 +2274,7 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 			volume = LinearGainToDB(outerGain);
 
 			IDirectSound3DBuffer_SetConeOutsideVolume((LPDIRECTSOUND3DBUFFER)ALSource->uservalue2, volume,DS3D_IMMEDIATE);
+
 			ALSource->update1 &= ~CONEOUTSIDEVOLUME;
 			if (ALSource->update1 == 0)
 				return;
@@ -2171,6 +2301,7 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 		{
 			Relative = ALSource->relative ? DS3DMODE_HEADRELATIVE : DS3DMODE_NORMAL;
 			IDirectSound3DBuffer_SetMode((LPDIRECTSOUND3DBUFFER)ALSource->uservalue2,Relative,DS3D_IMMEDIATE);
+
 			ALSource->update1 &= ~MODE;
 			if (ALSource->update1 == 0)
 				return;
@@ -2187,6 +2318,7 @@ void UpdateSource(ALCcontext *ALContext, ALsource *ALSource)
 			outerAngle = (ALuint)ALSource->param[AL_CONE_OUTER_ANGLE-AL_CONE_INNER_ANGLE].data.f;
 
 			IDirectSound3DBuffer_SetConeAngles((LPDIRECTSOUND3DBUFFER)ALSource->uservalue2,innerAngle,outerAngle,DS3D_IMMEDIATE);
+
 			ALSource->update1 &= ~CONEANGLES;
 			if (ALSource->update1 == 0)
 				return;
@@ -2428,7 +2560,7 @@ void UpdateListener(ALCcontext *ALContext)
 }
 
 
-ALCAPI ALCdevice* ALCAPIENTRY alcOpenDevice(ALCubyte *deviceName)
+ALCAPI ALCdevice* ALCAPIENTRY alcOpenDevice(const ALCubyte *deviceName)
 {
 	DSBUFFERDESC DSBDescription;
 	WAVEFORMATEX OutputType;
@@ -2441,6 +2573,7 @@ ALCAPI ALCdevice* ALCAPIENTRY alcOpenDevice(ALCubyte *deviceName)
 	ALboolean bUseDS;
 	ALboolean bUseDS3D;
 	ALboolean bDeviceFound = AL_FALSE;
+//	ALuint numStereo, numMono;
 	ALint numSources;
 
 	bUseDS = AL_FALSE;
@@ -2511,14 +2644,13 @@ ALCAPI ALCdevice* ALCAPIENTRY alcOpenDevice(ALCubyte *deviceName)
 										// Check that is an accelerated DS device
 										if (!(dsCaps.dwFlags & DSCAPS_EMULDRIVER))
 										{
-											numSources = (GetMaxNum3DMonoBuffers(device->DShandle) - 1);
+											numSources = GetMaxNum3DMonoBuffers(device->DShandle);
 
 											// To enable the 'DirectSound3D' device, the audio card MUST support
 											// at least 16 voices.  (If not, then the device selection drops through
 											// to the 'DirectSound' device).
 											if (numSources >= 16)
 											{
-												device->CreationFlag = DSBCAPS_LOCHARDWARE;
 												device->MaxNoOfSources = numSources;
 
 												strcpy(device->szDeviceName, "DirectSound3D");
@@ -2579,7 +2711,7 @@ ALCAPI ALCdevice* ALCAPIENTRY alcOpenDevice(ALCubyte *deviceName)
 								memset(&DSBDescription,0,sizeof(DSBUFFERDESC));
 								DSBDescription.dwSize=sizeof(DSBUFFERDESC);
 								DSBDescription.dwFlags=DSBCAPS_GLOBALFOCUS|DSBCAPS_GETCURRENTPOSITION2;
-								DSBDescription.dwBufferBytes=32768;
+								DSBDescription.dwBufferBytes=OUTPUT_BUFFER_SIZE;
 								DSBDescription.lpwfxFormat=&OutputType;
 								if (IDirectSound_CreateSoundBuffer(device->DShandle,&DSBDescription,&device->DSsbuffer,NULL)==DS_OK)
 								{
@@ -2833,7 +2965,7 @@ DWORD WINAPI ThreadProc(LPVOID lpParameter)
 
 	Determine if there is an extension called extName available
 */
-ALCAPI ALCboolean ALCAPIENTRY alcIsExtensionPresent(ALCdevice *device,ALCubyte *extName)
+ALCAPI ALCboolean ALCAPIENTRY alcIsExtensionPresent(ALCdevice *device, const ALCubyte *extName)
 {
 	ALsizei i=0;
 
@@ -2848,7 +2980,7 @@ ALCAPI ALCboolean ALCAPIENTRY alcIsExtensionPresent(ALCdevice *device,ALCubyte *
 
 	Get the function address for given function extension
 */
-ALCAPI ALCvoid *  ALCAPIENTRY alcGetProcAddress(ALCdevice *device,ALCubyte *funcName)
+ALCAPI ALCvoid *  ALCAPIENTRY alcGetProcAddress(ALCdevice *device, const ALCubyte *funcName)
 {
 	ALsizei i=0;
 
@@ -2864,11 +2996,48 @@ ALCAPI ALCvoid *  ALCAPIENTRY alcGetProcAddress(ALCdevice *device,ALCubyte *func
 	Get the value for a particular ALC Enumerated Value
 	Calls alGetEnumValue in alExtension.c to process request
 */
-ALCAPI ALCenum ALCAPIENTRY alcGetEnumValue(ALCdevice *device,ALCubyte *enumName)
+ALCAPI ALCenum ALCAPIENTRY alcGetEnumValue(ALCdevice *device, const ALCubyte *enumName)
 {
 	return alGetEnumValue(enumName);
 }
 
+
+/*
+	To workaround a bug in certain EAX supporting drivers, we need to create a dummy 3D buffer
+	and hold on to it for the life of the application.
+*/
+void EAXFix(ALCcontext *context)
+{
+	ALuint alDummySource;
+	ALboolean	bEAX20 = AL_FALSE;
+
+	if (context->alPrivateSource)
+		return;
+
+	// Generate a dummy Source
+	alGenSources(1, &alDummySource);
+
+	// Query for EAX 2.0 Support (this function will make an EAX 2.0 Set() call if EAX 2.0 is detected)
+	bEAX20 = alIsExtensionPresent((ALubyte*)"EAX2.0");
+
+	if (bEAX20)
+	{
+		// Need to generate Permanent Source
+		alGenSources(1, &(context->alPrivateSource));
+	}
+
+	// Delete dummy Source
+	alDeleteSources(1, &alDummySource);
+
+	// Update Context Source variables
+	if (context->alPrivateSource)
+	{
+		context->Source = NULL;
+		context->SourceCount = 0;
+		context->Device->MaxNoOfSources--;
+	}
+}
+
 /*
 	SetGlobalRolloffFactor
 
