@@ -1,11 +1,10 @@
 // OpenALDemo.cpp : Defines the entry point for the console application.
 //
 
-
-
 #include <stdio.h>
 #include <conio.h>
 #include <stdlib.h>
+#include "al\al.h"
 #include "al\alc.h"
 #include "al\alut.h"
 #include <memory.h>
@@ -22,8 +21,6 @@ const GUID DSPROPSETID_EAX20_ListenerProperties
 const GUID DSPROPSETID_EAX20_BufferProperties
 				= { 0x306a6a7, 0xb224, 0x11d2, {0x99, 0xe5, 0x0, 0x0, 0xe8, 0xd8, 0xc7, 0x22 } };
 
-
-
 #define NUM_BUFFERS 7	// Number of buffers to be Generated
 
 typedef struct
@@ -96,6 +93,11 @@ int main(int argc, char* argv[])
 	ALboolean loop;
 	ALCcontext *Context;
 	ALCdevice *Device;
+	char deviceName[256];
+	char *defaultDevice;
+	char *deviceList;
+	char *devices[12];
+	int numDevices, numDefaultDevice, i;
 
 	ALfloat listenerPos[]={0.0,0.0,0.0};
 	ALfloat listenerVel[]={0.0,0.0,0.0};
@@ -106,7 +108,48 @@ int main(int argc, char* argv[])
 	// Initialize Open AL manually
 
 	//Open device
-	Device = alcOpenDevice((ALubyte*)"DirectSound3D");
+	strcpy(deviceName, "");
+	if (alcIsExtensionPresent(NULL, (ALubyte*)"ALC_ENUMERATION_EXT") == AL_TRUE) { // try out enumeration extension
+		defaultDevice = (char *)alcGetString(NULL, ALC_DEFAULT_DEVICE_SPECIFIER);
+		deviceList = (char *)alcGetString(NULL, ALC_DEVICE_SPECIFIER);
+		for (numDevices = 0; numDevices < 12; numDevices++) {devices[numDevices] = NULL;}
+		for (numDevices = 0; numDevices < 12; numDevices++) {
+			devices[numDevices] = deviceList;
+			if (strcmp(devices[numDevices], defaultDevice) == 0) {
+				numDefaultDevice = numDevices;
+			}
+			deviceList += strlen(deviceList);
+			if (deviceList[0] == 0) {
+				if (deviceList[1] == 0) {
+					break;
+				} else {
+					deviceList += 1;
+				}
+			}
+		}
+		if (devices[numDevices] != NULL) {
+			numDevices++;
+			printf("\nEnumeration extension found -- select an output device:\n");
+			printf("0. NULL Device (Default)\n");
+			for (i = 0; i < numDevices; i++) {
+				printf("%d. %s\n", i + 1, devices[i]);
+			}
+			printf("\n\n");
+			do {
+				ch = _getch();
+				i = atoi(&ch);
+			} while ((i < 0) || (i > numDevices));
+			if ((i != 0) && (strlen(devices[i-1]) < 256)) {
+				strcpy(deviceName, devices[i-1]);
+			}
+		}
+	}
+
+	if (strlen(deviceName) == 0) {
+		Device = alcOpenDevice(NULL); // this is supposed to select the "preferred device"
+	} else {
+		Device = alcOpenDevice((ALubyte*)deviceName); // have a name from enumeration process above, so use it...
+	}
 
 	if (Device == NULL)
 	{
@@ -131,7 +174,7 @@ int main(int argc, char* argv[])
 		exit(-1);
 	}
 
-	// Clear Error Codes
+	// Clear Error Code
 	alGetError();
 	alcGetError(Device);
 
@@ -606,15 +649,12 @@ ALvoid PositionTest(ALvoid)
 	ALuint	source[2];
 	ALbyte	ch;
 
-	ALfloat source0Pos[]={ -4.0, 0.0, 4.0};	// Behind and to the left of the listener
+	ALfloat source0Pos[]={ -2.0, 0.0, 2.0};	// Behind and to the left of the listener
 	ALfloat source0Vel[]={ 0.0, 0.0, 0.0};
 
 	ALfloat source1Pos[]={ 2.0, 0.0, -2.0};	// Front and to the right of the listener
 	ALfloat source1Vel[]={ 0.0, 0.0, 0.0};
 
-	// temp set distance model
-	alDistanceModel(AL_INVERSE_DISTANCE);
-
 	alGenSources(2,source);
 	if ((error = alGetError()) != AL_NO_ERROR)
 	{
